#!/usr/bin/perl
package virtualmin_wp_workbench;

use File::Basename;

=head1 configure-script.pl

Configure the web app script using the workbench API

This command can be used to modify web application settings, perform backups,
make clones and carry out other administrative tasks, if supported by the
application.

See the usage by calling the '--help' flag with the script function for more
information on how to use this command, as it's specific to the web app
script type.

=cut

if (!$module_name) {
	$main::no_acl_check++;
	$ENV{'WEBMIN_CONFIG'} ||= "/etc/webmin";
	$ENV{'WEBMIN_VAR'} ||= "/var/webmin";
	my $pwd = dirname($0);
	$0 = "$pwd/configure-script.pl";
	require "$pwd/virtualmin-wp-workbench-lib.pl";
	}

# Pre-process args to get web app name
our $web_app_name;
for (my $i=0; $i<@ARGV; $i++) {
	if ($ARGV[$i] eq '--script-type' && $i+1 < @ARGV) {
		$web_app_name = $ARGV[$i+1];
		last;
		}
	}

# Check for missing --name parameter
if (!$web_app_name) {
	&usage("Missing script type name");
	}

# Check if the script type is supported
my $workbench_script_usage_func = "script_${web_app_name}_kit_usage";
my $workbench_script_cli = "script_${web_app_name}_kit_cli";
if (!defined(&$workbench_script_cli)) {
	&usage("Script type of '$web_app_name' does not support workbench API");
	}

# Parse common command-line flags
&virtual_server::parse_common_cli_flags(\@ARGV);

# Call the script-specific function
$workbench_script_cli->(\@ARGV);

# Expandable usage function
sub usage
{
print "$_[0]\n\n" if ($_[0]);
print "Configure the web app script using the workbench API\n\n";
print "virtualmin configure-script --script-type name";
if (defined(&$workbench_script_usage_func)) {
	$workbench_script_usage_func->($web_app_name);
	}
else {
	print "\n";
	}
exit(1);
}
