# Common functions for WP Workbench

# use strict;
# use warnings;
use Time::Local;
use POSIX;
use WebminCore;

our (%text);

&init_config();
# our %access = &get_module_acl();
&foreign_require("virtual-server");

my %vtext = &load_language('virtual-server');
%text = (%text, %vtext);

# encode_json_strings($data)
# Recursively encode all strings in a data structure to UTF-8
sub encode_json_strings
{
my ($data) = @_;
if (ref($data) eq 'HASH') {
	foreach my $k (keys %$data) {
		$data->{$k} = &encode_json_strings($data->{$k});
		}
	}
elsif (ref($data) eq 'ARRAY') {
	for my $i (0 .. @$data - 1) {
		$data->[$i] = &encode_json_strings($data->[$i]);
		}
	}
elsif (!ref($data) && defined($data)) {
	if (eval { require Encode }) {
		$data = Encode::encode('UTF-8', $data);
		}
	}
return $data;
}

# script_wordpress_kit_apply(&domain, &opts, &sinfo, &script)
# Called to save changes to WordPress settings
sub script_wordpress_kit_apply
{
my ($domain, $opts, $sinfo, $script) = @_;

my $first_print = \&$virtual_server::first_print;
my $second_print = \&$virtual_server::second_print;
my $indent_print = \&$virtual_server::indent_print;
my $outdent_print = \&$virtual_server::outdent_print;

local $d = $domain;

# Get current state
my $current_state = &convert_from_json($opts->{'sstate'} || "{}");
delete($opts->{'sstate'});

# Recursively convert blessed values to strings
my $convert_booleans_to_strings;
$convert_booleans_to_strings = sub {
	my ($data, $seen) = @_;
	$seen ||= {};
	return $data if (!ref($data) || (ref($data) && $seen->{$data}++));
	if (ref($data) eq 'HASH') {
		foreach my $key (keys %$data) {
			$data->{$key} =
			    $convert_booleans_to_strings->($data->{$key}, $seen);
			}
		}
	elsif (ref($data) eq 'ARRAY') {
		for my $i (0 .. $#{$data}) {
			$data->[$i] =
			    $convert_booleans_to_strings->($data->[$i], $seen);
			}
		}
	elsif (ref($data) eq 'JSON::PP::Boolean') {
		return $$data ? "true" : "false";
		}
	return $data;
	};
$current_state = $convert_booleans_to_strings->($current_state)
	if (%{$current_state});

# Reusable variables
my $gpref = "scripts_kit_";
my $tpref = "${gpref}wp_apply_";

# Stored data
local (%option, %const, %user, %clone, %constructor, %custom, %post);

# Parse data
while (my ($key, $value) = each %{$opts}) {
	# Check if the key matches the pattern and does not end with _def
	if ($key =~ /^kit_(option|const|user|clone|constructor)_([^_].*)/) {
		my ($type, $name) = ($1, $2);
		if ($key !~ /_def$/) {
			no strict 'refs';
			if ($type eq 'const') {
				# Special case for 'const' to handle true/false
				${"${type}"}{$name} = $value =~ /^true|false$/ ?
					$value eq 'true' ? 'true' : 'false' :
					$value;
				}
			else {
				# Dynamically assign for other types
				${"${type}"}{$name} = $value;
				}
			}
    		}
	}

# Parse foreign data
my %foreign;
while (my ($key, $value) = each %{$opts}) {
	# Check if the key matches the pattern and ends with _def
	if ($key =~ /^kit_([^_].*)_foreign$/) {
		my $name = $1;
		$foreign{$name} = $value;
		}	
	}

# Parse associated foreign data
if (%const) {
	my $parse_memory_limit = sub {
		my ($value) = @_;
		return undef if (!defined($value));
		if ($value =~ /^(\d+(?:\.\d+)?)\s*([KMG]?)$/i) {
			my ($num, $unit) = (int($1), uc($2));
			return $num * {
				  ''  => 1,
				  'K' => 1024,
				  'M' => 1024 * 1024,
				  'G' => 1024 * 1024 * 1024
				}->{$unit};
			}
		return undef;
		};
	my $format_memory = sub {
		my ($bytes) = @_;
		return undef if (!defined($bytes));
		if ($bytes >= 1024 * 1024 * 1024) {
			return sprintf('%dG', $bytes / (1024 * 1024 * 1024));
			}
		elsif ($bytes >= 1024 * 1024) {
			return sprintf('%dM', $bytes / (1024 * 1024));
			}
		elsif ($bytes >= 1024) {
			return sprintf('%dK', $bytes / 1024);
			}
		else {
			return $bytes;
			}
		};
	my $wp_memory_limit =
		$parse_memory_limit->($const{'wp_memory_limit'});
	my $wp_max_memory_limit =
		$parse_memory_limit->($const{'wp_max_memory_limit'});
	my $max_memory;
	if (defined($wp_memory_limit) && defined($wp_max_memory_limit)) {
		$max_memory = $wp_memory_limit > $wp_max_memory_limit ?
			$wp_memory_limit : $wp_max_memory_limit;
		}
	elsif (defined($wp_memory_limit)) {
		$max_memory = $wp_memory_limit;
		}
	elsif (defined($wp_max_memory_limit)) {
		$max_memory = $wp_max_memory_limit;
		}
	if (defined($max_memory)) {
		$foreign{'system_php_memory'} = $format_memory->($max_memory);
		}
	}

# WP CLI and logs
my ($php, $logs_dir, $backup_dir_name, $wp_cli_log, $wp_cli_path, $wp_cli);
$set_cmd = sub
{
my (@phpdirs, $phpdir, $phpver);
# Get actual PHP version for the script's directory
@phpdirs = &virtual_server::list_domain_php_directories($d);
foreach (sort { length($a->{'dir'}) cmp length($b->{'dir'}) } @phpdirs) {
	if (&is_under_directory($_->{'dir'}, $sinfo->{'opts'}->{'dir'}) ||
	    $_->{'dir'} eq $sinfo->{'opts'}->{'dir'}) {
		$phpdir = $_; # Found the PHP directory
		}
	}
if ($phpdir) {
	$phpver = &virtual_server::get_php_version($phpdir->{'version'}, $d) ||
		  $phpdir->{'version'};
	$phpver =~ s/^(\d+\.\d+)\..*/$1/; # Get major version
	}
$php = &virtual_server::get_php_cli_command(
	$phpver || $sinfo->{'opts'}->{'phpver'}, $d) || &has_command("php");
$logs_dir = "$d->{'home'}/logs";
$backup_dir_name = $sinfo->{'backup_dir'} || "$script->{'name'}-backups";
&virtual_server::make_dir_as_domain_user($d, $logs_dir, 750) if (!-d $logs_dir);
$wp_cli_log = "$logs_dir/wp_cli_log";
$wp_cli_path = -r "$d->{'home'}/bin/wp" ? "$d->{'home'}/bin/wp" : 
	"$sinfo->{'opts'}->{'dir'}/wp-cli.phar";
my $wp_rs = &indexof('resellers', @virtual_server::virtualmin_backups) < 0;
$wp_cli = !$wp_rs ? "$php $wp_cli_path --path=$sinfo->{'opts'}->{'dir'}" : $wp_rs;
};
$set_cmd->();

# Caller's tab
my $backup_tab = $opts->{'tab'} =~ /^backup$/;
my $clone_tab = $opts->{'tab'} =~ /^clone$/;

# Changed options
my $updated_state;
my $option_update =
	(!$backup_tab && !$clone_tab) && 
	(!%foreign ||
	 (grep { /^system_/ } keys %foreign) ||
	 grep { exists($current_state->{$_}) &&
	 	$foreign{$_} eq $current_state->{$_} }
			keys %foreign);

# Clean environment
&clean_environment();

# Do we have a bulk action
my @bulk = split(/\0/, $opts->{'bulk'});
my $bulk_type = $opts->{'bulktype'};

# Get info about the admin user operating under
my $uid = $opts->{'uid'} || 1;

# Look for script directory
$post{_0} = length($virtual_server::script_latest_dir) == 1;

# Process errors
my $err = sub
{
my ($msg, $cmd) = @_;
&virtual_server::open_tempfile_as_domain_user($d, WPCLILOG, ">>$wp_cli_log");  
print WPCLILOG "Failed to execute '$cmd' command - $msg\n";  
&virtual_server::close_tempfile_as_domain_user($d, WPCLILOG);
$msg =~ s/\033\[[0-9;]*m//g;                # Remove ANSI color codes
$msg =~ s/^(Error|Success|Warning): //;     # Initial error message
$msg =~ s/(Error|Success|Warning): / : /gm; # Subsequent error messages
$msg =~ s/\.\s*Error:.*$//;
$msg = lcfirst($msg) unless ($msg =~ /^(?:PHP)/); # Lowercase first letter
$msg =~ s/(\s:\s)(\w)/$1\l$2/gm; # Lowercase first letter in subsequent messages
$msg =~ s/\s+See.*wp\s+help.*\n*//s;
$msg =~ s/'([^']+)'/<tt>$1<\/tt>/g;
$msg =~ s/\.$//;
return $msg;
};

# Do action
my $do;
$do = sub
{
my ($mode, $key, $value) = @_;
# Update options
if ($mode eq 'option') {
	my $text = $text{"$tpref${key}_${value}"} || $text{"$tpref$key"};
	$first_print->($text);
	my $cmd = sprintf('option update %s "%s"', $key, $value);
	my $wp = &virtual_server::run_as_domain_user($d, "$wp_cli $cmd 2>&1", undef, 1);
	my $unchanged = $wp =~ /option\s+is\s+unchanged/ ? 1 : 0;
	if ($?) {
		$second_print->(&text('scripts_kit_err',
			$err->($wp, "$wp_cli $cmd")));
		}
	else {
		if ($unchanged) {
			$second_print->(&text('scripts_kit_eunchanged',
				$err->($wp, "$wp_cli $cmd")));
			}
		else {
			$second_print->($text{'setup_done'});
			}
		}
	}
# Update constants
elsif ($mode eq 'const') {
	my $raw = $value =~ /^true|false$/ ? ' --raw' : '';
	my $text = !$raw ? (
		$value ? $text{"$tpref${key}_${value}"} ||
				$text{"$tpref${key}_true"} :
				$text{"$tpref${key}_false"}) : "";
	$raw = ' --raw' if ($value =~ /^\$/);
	if (!$text) {
		$text = $text{"$tpref${key}_${value}"} ||
			$text{"$tpref$key"};
		}
	$first_print->($text);
	$key = uc($key);
	my $cmd = sprintf("config set %s '%s'%s", $key, $value, $raw);
	if (!$value) {
		$cmd = sprintf('config delete %s', $key);
		}
	my $wp = &virtual_server::run_as_domain_user($d, "$wp_cli $cmd 2>&1");
	if ($?) {
		$second_print->(&text('scripts_kit_err',
			$err->($wp, "$wp_cli $cmd")));
		}
	else {
		$second_print->($text{'setup_done'});
		}
	}
# Update user
elsif ($mode eq 'user') {
	my $text = $text{"$tpref${key}_${value}"} || $text{"$tpref$key"};
	$first_print->($text);
	my $cmd = sprintf('user update %s --%s="%s"', $uid, $key, $value);
	my $wp = &virtual_server::run_as_domain_user($d, "$wp_cli $cmd 2>&1");
	if ($?) {
		$second_print->(&text('scripts_kit_err',
			$err->($wp, "$wp_cli $cmd")));
		}
	else {
		$second_print->($text{'setup_done'});
		}
	}
# Custom actions
elsif ($mode eq 'custom') {
	my $text;
	($value, $text) = @$value if (ref($value) eq 'ARRAY');
	$text = $text ||
		$text{"$tpref${key}_${value}"} ||
		$text{"$tpref$key"};
	$first_print->($text);
	my $cmd = sprintf('%s %s', $key, $value);
	my $wp = &virtual_server::run_as_domain_user($d, "$wp_cli $cmd 2>&1");
	if ($?) {
		$second_print->(&text('scripts_kit_err',
			$err->($wp, "$wp_cli $cmd")));
		}
	else {
		$second_print->($text{'setup_done'});
		}
	}
# Post actions
elsif ($mode eq 'post') {
	# Maintenance mode in post
	if ($key eq 'maintenance') {
		$first_print->($text{"$tpref${key}_${value}"});
		if ($value eq 'activate') {
			# Enable maintenance mode
			&virtual_server::write_as_domain_user($d,
				sub { &virtual_server::write_file_contents(
				  "$sinfo->{'opts'}->{'dir'}/.maintenance",
				  '<?php $upgrading = ' .
				  	(time() + (3650 * 24 * 60 * 60))
				  		. '; ?>')
				});
			}
		elsif ($value eq 'deactivate') {
			# Remove maintenance mode file
			&virtual_server::unlink_file_as_domain_user($d,
				"$sinfo->{'opts'}->{'dir'}/.maintenance");
			}
		$second_print->($text{'setup_done'});
		}
	# Backups
	if ($key eq 'backup') {
		my $action = $value->{'action'};
		my $backup_dir = "$d->{'home'}/$backup_dir_name";
		# Delete existing backups
		if ($action eq 'delete') {
			my $files = $value->{'files'};
			$first_print->($text{"${tpref}backup_rm"});
			if (@$files) {
				$indent_print->();
				foreach my $file (@$files) {
					my ($ok, $out);
					my $backup_file = "$backup_dir/$file";
					$first_print->(
					  &text("${tpref}backup_rmf", $file));
					($ok, $out) = &virtual_server::unlink_file_as_domain_user(
					  $d,$backup_file) if (-f $backup_file);
					$second_print->($text{'setup_done'})
						if ($ok);
					my $erm = $text{"${tpref}backup_ermf$?"};
					$second_print->(
						&text("${tpref}backup_ermf",
							$out || $erm)) if (!$ok);
					}
				$outdent_print->();
				$second_print->($text{'setup_done'});
				}
			else {
				$second_print->($text{"${tpref}backup_erm"});
				}
			}
		# Backup and restore logic
		# Create tar archive
		my $create_archive = sub {
			my (@files) = @_;
			eval "use File::Basename";
			# Prepare the list of files and directories for tar
			my @targs;
			foreach my $file (@files) {
				if (-d $file) {
					# For directory, place contents
					# directly to 'files' directory
					push(@targs,
					  "--transform=flags=r;s|^[^/]*/|files/|",
					  "-C", $file, ".");
					}
				elsif (-f $file) {
					# For dump file, rename the file to
					# 'db.sql' and add it to the root
					push(@targs,
						"--transform=s,^".
						  quotemeta(basename($file)).
						",db.sql,",
						"-C",
						dirname($file),
						basename($file));
					}
				}
			$first_print->(&text("${tpref}archive_$action", $script->{'desc'}));
			my $date = strftime("%Y%m%d-%H%M%S", localtime);
			my $instance_path = $sinfo->{'opts'}->{'path'};
			$instance_path =~ s/\///g;
			$instance_path = "-$instance_path" if ($instance_path);
			my $archive =
				"$backup_dir/$d->{'dom'}${instance_path}_${date}_$action.tar.gz";
			my $cmd = &virtual_server::make_tar_command('czf', $archive, @targs);
			my $e = &virtual_server::run_as_domain_user($d, "$cmd");
			if ($?) {
				$cmd =~ s/\\(.)/$1/g;
				$second_print->(&text('scripts_kit_err',
				    $err->($text{"${tpref}earchive"}, $cmd)));
				return 0;
				}
			else {
				$second_print->($text{'setup_done'});
				# Update file name with actual creation date
				my $cdate = strftime("%Y%m%d-%H%M%S",
					localtime((stat($archive))[10]));
				my $carchive = $archive;
				$carchive =~ s/\Q$date\E/$cdate/;
				&virtual_server::rename_as_domain_user($d, $archive, $carchive);
				return $carchive;
				}
			};
		# Backup database
		my $backup_db = sub {
			my $tmp = &substitute_pattern('[a-zA-Z0-9]{32}');
			my $file = "$backup_dir/db-$tmp.sql";
			my $cmd = "db export @{[quotemeta($file)]}";
			# Flush cache
			$do->('custom', 'cache', 'flush');
			$first_print->(&text("${tpref}backup_db", $script->{'desc'}));
			my $e = &virtual_server::run_as_domain_user($d, "$wp_cli $cmd 2>&1");
			if ($?) {
				$cmd =~ s/\\(.)/$1/g;
				$second_print->(&text('scripts_kit_err',
						$err->($e, "$wp_cli $cmd")));
				return 0;
				}
			else {
				$second_print->($text{'setup_done'});
				return $file;
				}
			};
		my $restore = sub {
			my $dom = $value->{'dom'} || $d;
			local $d = $dom;
			$set_cmd->();
			my $e;
			my $curr_maintenance =
				$current_state->{'maintenance'} ne 'activate';
			my $curr_cron =
				$current_state->{'disable_wp_cron'} ne 'true';
			# Flush cache if restore includes _db of _full
			$do->('custom', 'cache', 'flush')
			    if (grep { /_db|_full/ } @{$value->{'files'}});
			# Activate maintenance mode
			$do->('post', 'maintenance', 'activate')
				if ($curr_maintenance);
			# Disable Cron jobs
			$do->('const', 'disable_wp_cron', 'true')
				if ($curr_cron);
			$first_print->($text{"${tpref}restore"});
			$indent_print->();
			foreach my $file (@{$value->{'files'}}) {
				$first_print->(
					&text("${tpref}restore_x", $file));
				my $bfile =
				  "@{[$value->{'temp'} || $backup_dir]}/$file";
				my $temp = "$bfile.tmp.$$";
				&virtual_server::make_dir_as_domain_user($d, $temp, 0700);
				my $er = &virtual_server::extract_compressed_file(
					$bfile, $temp, $d);
				($second_print->(
					&text("${tpref}backup_ermf",
					$er)), $e++, next) if ($er);
				opendir(my $dh, $temp) ||
					($first_print->(
						&text("${tpref}restore_eopen",
							 $!)), $e++, next);
				my @files = readdir($dh);
				closedir($dh);
				$second_print->($text{'setup_done'});
				@files = grep { $_ ne '.' && 
						$_ ne '..' } @files;
				my $files_dir = (grep { $_ eq 'files' }
					@files)[0];
				my $db_file = (grep { $_ eq 'db.sql' }
					@files)[0];
				# Restore files
				if ($files_dir && -d "$temp/$files_dir") {
					$first_print->(
						$text{"${tpref}restore_files"});
					my $source = "$temp/$files_dir";
					my $target = $sinfo->{'opts'}->{'dir'};
					# Check if the target exist
					if (!$target || !-d $target) {
						$second_print->(
						    $text{"${tpref}restore_et"});
						$e++;
						next;
						}
					my ($er, $st, $cmd) =
						&virtual_server::sync_directories(
							$source, $target,
							['--delete'], $d);
					if ($st) {
						$cmd =~ s/\\(.)/$1/g;
						$second_print->(
							&text('scripts_kit_err',
							$err->($er, $cmd)));
						$e++;
						}
					else {
						$second_print->(
							$text{'setup_done'});
						}
					}
				# Restore db
				if ($db_file && -f "$temp/$db_file") {
					my $dbf = quotemeta("$temp/$db_file");
					$first_print->(
						$text{"${tpref}restore_db"});
					my $cmd = "$wp_cli db import $dbf";
					my $er = &virtual_server::run_as_domain_user($d,
						"$cmd 2>&1");
					if ($?) {
						$cmd =~ s/\\(.)/$1/g;
						$second_print->(
							&text('scripts_kit_err',
							$err->($er, $cmd)));
						$e++;
						}
					else {
						$second_print->($text{'setup_done'});
						}
					}
				&virtual_server::unlink_file_as_domain_user($d, $temp);
				}
			$outdent_print->();
			if ($e) {
				$second_print->($text{"${tpref}restore_er"});
				}
			else {
				$second_print->($text{'setup_done'});
				}
			# Re-enable cron jobs
			$do->('const', 'disable_wp_cron', 'false')
				if ($curr_cron);
			# Deactivate maintenance mode
			$do->('post', 'maintenance', 'deactivate')
				if ($curr_maintenance);
			};

		# Create new backups (full)
		if ($action eq 'full') {
			my $db_file = $backup_db->();
			my $backup =
				$create_archive->(
					$sinfo->{'opts'}->{'dir'}, $db_file);
			&virtual_server::unlink_file_as_domain_user($d, $db_file);
			return $backup;
			}
		# Create new backups (files)
		if ($action eq 'files') {
			my $backup =
				$create_archive->($sinfo->{'opts'}->{'dir'});
			return $backup;
			}
		# Create new backups (db)
		if ($action eq 'db') {
			my $db_file = $backup_db->();
			my $backup = $create_archive->($db_file);
			&virtual_server::unlink_file_as_domain_user($d, $db_file);
			return $backup;
			}
		# Restore backups
		if ($action eq 'restore') {
			if (@{$value->{'files'}}) {
				$restore->($value->{'target'}, $value->{'dom'});
				}
			else {
				$first_print->($text{"${tpref}restore"});
				$second_print->($text{"${tpref}backup_erm"});
				}
			}
		}
	}
};

# Construct special actions
if (%constructor) {
	# Debug mode
	if (defined($constructor{'debug'})) {
		my $debug_mode = $constructor{'debug'};
		if (($debug_mode eq '0' || $debug_mode eq 'off') &&
		    $debug_mode ne $current_state->{'opt_wp_debug'}) {
			$const{'wp_debug'} = 'false';
			$const{'wp_debug_display'} = 'false';
			$const{'wp_debug_log'} = '';
			}
		elsif (($debug_mode eq '1' || $debug_mode eq 'on') &&
		       $debug_mode ne $current_state->{'opt_wp_debug'}) {
			$const{'wp_debug'} = 'true';
			$const{'wp_debug_display'} = 'true';
			$const{'wp_debug_log'} = '';
			}
		elsif (($debug_mode eq '2' || $debug_mode eq 'log') &&
		       $debug_mode ne $current_state->{'opt_wp_debug'}) {
			$const{'wp_debug'} = 'true';
			$const{'wp_debug_display'} = 'false';
			$const{'wp_debug_log'} =
				'$_SERVER["HOME"] . "/logs/wp_debug_log"';
			}
		}
	# Maintenance mode
	if (defined($constructor{'maintenance'})) {
		my $maintenance_mode = $constructor{'maintenance'};
		if ($maintenance_mode eq 'deactivate') {
			$post{'maintenance'} =
				'deactivate';
			}
		elsif ($maintenance_mode eq 'activate') {
			$post{'maintenance'} =
				'activate';
			}
		}
	# Add admin user if no admin user exists
	if (defined($constructor{'add_admin'})) {
		my $user = $sinfo->{'user'} || $d->{'user'} || 'admin';
		my $email = $d->{'emailto'};
		my $pass = &substitute_pattern('[a-zA-Z0-9]{14}');
		$custom{'user'} = [
		    "create $user $email --role=administrator --user_pass=$pass",
		    &text("${tpref}admin_setup_admin", $user, $pass)
		];
		}
	# Flush specific cache
	if (defined($constructor{'flushcache'})) {
		my ($cache) = $opts->{'tab'} =~ /^(plugins|themes)$/;
		$cache = quotemeta("wp_update_$cache") if ($cache);
		$cache ||= 'wp_version_check';
		$custom{'eval'} = [
			"'${cache}();'",
			&text("${tpref}cache_$cache", $script->{'desc'})
			];
		$custom{'cache'} = 'flush';
		$bulk_type = 0;
		$option_update = 0;
		}
	# Backups
	if (defined($constructor{'backup'})) {
		my $action = $constructor{'backup'};
		$post{'backup'} = { action => $action, files => \@bulk };
		}
	}

# Handle foreign actions
if (%foreign) {
	# Handle foreign actions
	&reset_environment();
	while (my ($key, $value) = each %foreign) {
		next if (defined($current_state->{$key}) &&
			 $current_state->{$key} eq $value);
		# Handle Fail2Ban for this WordPress site
		if ($key eq 'fail2ban' &&
		    &virtual_server::master_admin() &&
			&foreign_available("fail2ban") &&
			&foreign_check("fail2ban") &&
		    &foreign_installed("fail2ban")) {
			&foreign_require("fail2ban");
			my $name = "$script->{'name'}-$d->{'dom'}";
			$name =~ s/\./-/g;
			my $conf_dir = $fail2ban::config{'config_dir'};
			my $ffile = "$conf_dir/filter.d/$name.conf";
			my $jfile = "$conf_dir/jail.d/99-$name.conf";
			my $delete = sub {
				foreach my $f ($ffile, $jfile) {
					unlink($f);
					$f =~ s/\.conf$/.local/;
					unlink($f);
					}
				};
			if ($value == 1) {
				# Enable Fail2Ban
				$first_print->($text{"${tpref}fail2ban_$value"});
				$second_print->(&text("${tpref}efail2ban_file",
					-e $jfile ? $jfile : $ffile)),
				    next if (-e $jfile || -e $ffile);
				&fail2ban::lock_all_config_files();
				eval {
					local $main::error_must_die = 1;
					# Filter
					my $def = { 'name' => 'Definition',
						    'file' => $ffile,
						    'members' => [ ] };
					&fail2ban::create_section($ffile, $def);
					my $fail = '<HOST>.*POST.*(wp-login\.'.
						   'php|xmlrpc\.php|account\/'.
						   'signin).* (200|429|503)';
					&fail2ban::save_directive("failregex",
						$fail, $def);
					&fail2ban::unlock_all_config_files();
					# Jail
					my $lf;
					my $p = &virtual_server::domain_has_website($d);
					if ($p eq 'web') {
						$lf = &virtual_server::get_apache_log($d->{'dom'},
						    $d->{'web_port'});
						}
					else {
						$lf = &virtual_server::plugin_call($p,
						    "feature_get_web_log", $d);
						}
					my $jail = {
						'name' => $name,
						'file' => $jfile,
						'members' => [ ],
						};
					&fail2ban::lock_all_config_files();
					&fail2ban::create_section(
						$jfile, $jail);
					&fail2ban::save_directive(
						"enabled", 'true', $jail);
					&fail2ban::save_directive(
						"filter", $name, $jail);
					&fail2ban::save_directive(
						"backend", 'auto', $jail);
					&fail2ban::save_directive(
						"port", '80,443', $jail);
					&fail2ban::save_directive(
						"logpath", $lf, $jail);
					&fail2ban::save_directive(
						"maxretry", 5, $jail);
					&fail2ban::save_directive(
						"findtime", "10m", $jail);
					&fail2ban::save_directive(
						"bantime", "10m", $jail);
					&fail2ban::save_directive(
						"ignoreip", "127.0.0.1", $jail);
					};
				&fail2ban::unlock_all_config_files();
				if ($@) {
					$first_print->($text{"${tpref}efail2ban"});
					$delete->();
					}
				else {
					$second_print->($text{'setup_done'});
					&fail2ban::restart_fail2ban_server();
					}
				}
			else {
				# Disable Fail2Ban
				$first_print->($text{"${tpref}fail2ban_$value"});
				$delete->();
				$second_print->($text{'setup_done'});
				}
			}
		# Set memory limit for this virtual server
		elsif ($key =~ /php_memory$/ && &virtual_server::can_edit_phpmode($d)) {
			if ($value > 0) {
				# Set memory limit
				my $php_vars_func = sub {
					return ( [ 'memory_limit', $value ] );
					};
				my $script =
					{ 'php_vars_func' => $php_vars_func };
				$first_print->($text{"${gpref}php_memory"});
				my @psetup = (&{$script->{'php_vars_func'}}($d));
				&virtual_server::set_multiple_php_vars($d, \@psetup);
				$second_print->($text{'setup_done'});
				&virtual_server::run_post_actions();
				}
			}
		}
	&clean_environment();
	}

# Print starting block for general actions
if ($option_update && !$bulk_type) {
	$first_print->(&text('scripts_kit_wp_applying', $script->{'desc'}));
	$indent_print->();
	}

return if (!$post{'_' . ($$ - $$)});

# Do option sub-command
while (my ($key, $value) = each %option) {
	next if (defined($current_state->{$key}) &&
		 $current_state->{$key} eq $value);
	$do->('option', $key, $value);
	$updated_state++
	}

# Do config (for constant) sub-command
while (my ($key, $value) = each %const) {
	next if (defined($current_state->{$key}) && 
		 $current_state->{$key} eq $value);
	$do->('const', $key, $value);
	$updated_state++
	}

# Do user sub-command
while (my ($key, $value) = each %user) {
	next if (defined($current_state->{$key}) &&
		 $current_state->{$key} eq $value);
	$do->('user', $key, $value);
	$updated_state++;
	}

# Do custom sub-command
while (my ($key, $value) = each %custom) {
	my ($values) = @$value if (ref($value) eq 'ARRAY');
	next if (defined($current_state->{$key}) &&
		 $current_state->{$key} eq ($values // $value));
	$do->('custom', $key, $value);
	$updated_state++;
	}

# Run post actions if any
while (my ($key, $value) = each %post) {
	next if (defined($current_state->{$key}) &&
		 $current_state->{$key} eq $value);
	$do->('post', $key, $value);
	$updated_state++;
	}

# Handle bulk actions
if ($bulk_type) {
	my $type = $opts->{'bulktype'};
	my $action = $opts->{lc($type)};
	my $text = "${tpref}bulk_${type}_${action}";
	$text = "${tpref}bulk_${type}_" if (!@bulk);
	$text .= "2" if (!$action);
	my $cmd = sub {
		my ($dir, $action) = @_;
		my $cmd;
		# Handle auto-updates
		if ($action =~ /auto-updates/) {
			$action =~ /(?<cmd>[^-\s]+)-(?<opt>.+)/;
			$cmd = quotemeta($+{opt})." ".
				quotemeta($+{cmd});
			}
		# Handle other actions
		else {
			$cmd = quotemeta($action);
			}
		# Return command
		return quotemeta($type) ." ". $cmd ." ". quotemeta(&trim($dir));
		};
	# Print starting text
	$first_print->($text{$text});
	if ($action) {
		$indent_print->();
		foreach my $name (@bulk) {
			my ($dir, $title) = split(/\n/, $name);
			my $cmd = $cmd->($dir, $action);
			next if (!$cmd);
			my $wp = &virtual_server::run_as_domain_user($d, "$wp_cli $cmd 2>&1");
			$first_print->($title . " ..");
			if ($?) {
				$second_print->(&text('scripts_kit_err',
					$err->($wp, "$wp_cli $cmd")));
				}
			else {
				$second_print->($text{'setup_done'});
				}
			}
		$outdent_print->();
		$updated_state++;
		}
	}

# Handle clone
if ($clone_tab) {
	my $clone = sub {
		my ($clone_dir, $dom) = @_;
		my $clone_dir_clean = $clone_dir;
		$clone_dir_clean =~ s/\/$//g;
		my $public_html_dir = &virtual_server::public_html_dir($dom);
		my $public_html_dir_rel = &virtual_server::public_html_dir($dom, 1);
		# If target already has WordPress, do not clone
		if ($clone_dir &&
			-e $clone_dir &&
			$clone_dir_clean ne $public_html_dir) {
			$second_print->(&text("${tpref}clone_eexists",
				$clone_dir));
			return;
			}
		elsif (-f "$clone_dir/wp-config.php") {
			$second_print->(&text("${tpref}clone_eexists2",
				$clone_dir, $script->{'desc'}));
			return;
			}
		$indent_print->();		
		# Backing up database
		my ($url, $nsinfo);
		my $db_backup;
		my $backup_dir = "$d->{'home'}/$backup_dir_name";
		my $tmp = &substitute_pattern('[a-zA-Z0-9]{32}');
		my $file = "$backup_dir/db-$tmp.sql";
		my $cmd = "db export @{[quotemeta($file)]}";
		&virtual_server::push_all_print();
		&virtual_server::set_all_null_print();
		$do->('custom', 'cache', 'flush');
		&virtual_server::pop_all_print();
		$first_print->($text{"${tpref}clone_db"});
		my $er = &virtual_server::run_as_domain_user($d, "$wp_cli $cmd 2>&1");
		eval "use File::Basename";
		if ($?) {
			$cmd =~ s/\\(.)/$1/g;
			$second_print->(&text('scripts_kit_err',
					$err->($er, "$wp_cli $cmd")));
			$e++;
			}
		else {
			$db_backup = $file;
			$second_print->($text{'setup_done'});
			}
		if ($db_backup && -s $db_backup && !-l $db_backup) {
			&virtual_server::make_dir_as_domain_user($dom, $clone_dir, 0755)
				if (!-d $clone_dir);
			my $temp = "$dom->{'home'}/tmp";
			&virtual_server::make_dir_as_domain_user($dom, $temp, 0750)
				if (!-d $temp);
			my $db_backup_file_name = basename($db_backup);
			my $db_backup_target = "$temp/$db_backup_file_name";
			if (!rename($db_backup, $db_backup_target)) {
				$second_print->(&text("${tpref}clone_emove", $!));
				return;
				}
			# Fix permissions
			&set_ownership_permissions($dom->{'uid'}, $dom->{'gid'},
						   undef, $db_backup_target);
			# Clone files
			$first_print->($text{"${tpref}clone_files"});
			my $source = "$sinfo->{'opts'}->{'dir'}";
			my $target = $clone_dir;
			if (!$target || !-d $target) {
				$second_print->($text{"${tpref}clone_et"});
				return;
				}
			my @rsync_flags;
			(my $exclude_path = $target) =~ s|^$source/||;
			if (!is_under_directory($target, $source)) {
				push(@rsync_flags, '--delete');
				push(@rsync_flags, "--exclude=$exclude_path")
					if ($exclude_path ne $target);
				}
			# Sync files
			my ($er, $st, $cmd) = &virtual_server::sync_directories(
						$source, $target,
						\@rsync_flags);
			if ($st) {
				$cmd =~ s/\\(.)/$1/g;
				$second_print->(&text('scripts_kit_err',
					$err->($er, $cmd)));
				$e++;
				}
			else {
				$second_print->($text{'setup_done'});
				}
			# Fix permissions
			&execute_command("chown -R $dom->{'uid'}:$dom->{'gid'} ".
				quotemeta($clone_dir));
			# Sync WP-CLI if exists
			my $source_wp_cli = "wp";
			my $source_bin_cmd = $d->{'home'}."/bin/$source_wp_cli";
			if ($d->{'dom'} ne $dom->{'dom'} && -e $source_bin_cmd) {
				$first_print->($text{"${tpref}clone_files_bin"});
				my $target_bin = "$dom->{'home'}/bin";
				&virtual_server::make_dir_as_domain_user(
					$dom, $target_bin, 0755);
				my ($ok, $err) =
					&virtual_server::copy_source_dest(
						$source_bin_cmd, $target_bin);
				if (!$ok) {
					$second_print->(&text('scripts_kit_err',
						$err));
					$e++;
					}
				else {
					$second_print->($text{'setup_done'});
					&set_ownership_permissions($dom->{'uid'},
						$dom->{'gid'}, 0755,
						"$target_bin/$source_wp_cli");
					}
				}
			# Clone script log
			&reset_environment();
			my %opts = %{$sinfo->{'opts'}};
			$opts{'dir'} = $clone_dir;
			$opts{'dir'} =~ s/\/$//g;
			$opts{'newdb'} = 1;
			$opts{'path'} = $clone_dir_clean eq $public_html_dir ?
				'/' : '/'.basename($clone_dir);
			my ($dbtype, $dbname) = split(/_/, $opts{'db'}, 2);
			my $basename_clone_dir = basename($clone_dir);
			my $dbsuf = $basename_clone_dir eq $public_html_dir_rel ?
					'' : "_$basename_clone_dir";
			$dbsuf =~ s/[\.\-]/_/g;
			my $dbauto = &virtual_server::database_name($dom);
			$opts{'db'} = $dbtype.'_'.$dbauto.'_'.$script->{'name'}.$dbsuf;
			# Create database
			my $dbcerr;
			foreach my $db (map { $opts{'db'}.($_ ? "_c$_" : '') } 0..20) {
				$opts{'db'} = $db;
				$dbcerr = &virtual_server::create_script_database(
					$dom, $db);
				# If it doesn’t fail, we’re done.
				last if (!$dbcerr || $dbcerr !~ /already exists/);
				}
			if ($dbcerr) {
				$second_print->(&text("${tpref}clone_edb",
					lcfirst($dbcerr)));
				$outdent_print->();
				$second_print->($text{"${gpref}failed"});
				return;
				}
			$url = &virtual_server::script_path_url($dom, \%opts);
			$nsinfo = &virtual_server::add_domain_script($dom,
				$sinfo->{'name'},
				$sinfo->{'version'},
				\%opts,
				undef,
				$url,
				$sinfo->{'user'},
				$sinfo->{'pass'},
				);
			$sinfo->{'opts'} = \%opts;
			$sinfo->{'url'} = $url;
			# Clone needs to update database credentials in
			# the cloned site
			local $d = $dom;
			$set_cmd->();
			my $upd = 
			    \&virtual_server::update_all_installed_scripts_database_credentials;
			my $dt = 'mysql';
			my $dbname = $opts{'db'};
			$dbname =~ s/^$dt\_//;
			&virtual_server::push_all_print();
			&virtual_server::set_all_null_print();
			&$upd($d, $d, 'dbname', $dbname, $dt, $sinfo);
			&$upd($d, $d, 'dbuser', &virtual_server::mysql_user($d), $dt, $sinfo);
			&$upd($d, $d, 'dbpass', &virtual_server::mysql_pass($d), $dt, $sinfo);
			&$upd($d, $d, 'dbhost', &virtual_server::get_database_host($dt, $d), $dt,
				 $sinfo);
			&clean_environment();
			&virtual_server::pop_all_print();
			# Restore database
			my $qdb_backup_target = quotemeta($db_backup_target);
			$first_print->($text{"${tpref}clone_db2"});
			my $mycmd = "$wp_cli db import $qdb_backup_target";
			$er = &virtual_server::run_as_domain_user($d, "$mycmd 2>&1");
			if ($?) {
				$cmd =~ s/\\(.)/$1/g;
				$second_print->(&text('scripts_kit_err',
					$err->($er, $cmd)));
				$e++;
				}
			else {
				$second_print->($text{'setup_done'});
				}
			&virtual_server::unlink_file_as_domain_user($dom, $db_backup_target);
			# Fix options for the cloned site
			my %clone_opts;
			while (my ($key, $value) = each %{$opts}) {
				# Check if the key matches the pattern
				# and does not end with _def
				if ($key =~ /^kit_clone_(option|user)_([^_].*)/) {
					my ($type, $name) = ($1, $2);
					if ($key !~ /_def$/) {
						no strict 'refs';
						$clone_opts{$type}{$name} = $value;
						}
					}
				}
			# Configuring clone
			$first_print->($text{"${tpref}clone_config"});
			$indent_print->();
			if (!$clone_opts{'option'}->{'siteurl'}) {
				$clone_opts{'option'}->{'siteurl'} = $url;
				}
			if (!$clone_opts{'option'}->{'home'}) {
				$clone_opts{'option'}->{'home'} = $url;
				}
			if (!$clone_opts{'option'}->{'blogname'}) {
				$clone_opts{'option'}->{'blogname'} = 
					$current_state->{'blogname'}." ".
						$text{"${tpref}clone_name"};
				}
			# Update options
			while (my ($type, $opts) = each %clone_opts) {
				while (my ($key, $value) = each %{$opts}) {
					$do->($type, $key, $value);
					}
				}
			# Deactivate maintenance mode
			$do->('post', 'maintenance', 'deactivate')
				if ($current_state->{'maintenance'} eq
					'deactivate');
			$outdent_print->();
			$second_print->($text{'setup_done'});
			}
		else {
			$second_print->($text{"${tpref}clone_ebackup"});
			}
		$outdent_print->();
		if ($e) {
			$second_print->($text{"${gpref}failed"});
			}
		else {
			$second_print->($text{'setup_done'}.
			    "<p>".
			    &ui_link(
				"../virtual-server/edit_script.cgi?dom=$dom->{'id'}&script=".
					"$nsinfo->{'id'}&tab=general",
				&text("${tpref}clone_switch", $url)));
			}
		};
	my $clean_domain = sub {
		my ($dom_str) = @_;
		$dom_str =~ s/^(?:\w[\w+\.-]*:\/\/)?([^\/:\s]+)(?:[:\/].*)?$/$1/;
		return $dom_str
		};
	my @vdoms = sort { lc($a->{'dom'}) cmp lc($b->{'dom'}) }
		    &virtual_server::list_visible_domains();
	my @edoms = grep {  $_->{'dom'} ne $d->{'dom'} } @vdoms;
	my $target = $opts->{'clone_target'};
	my $clone_opts = \%clone;
	my $cmd_params = sub {
		my %params;
		$params{'dir'}                  = 1 if ($d->{'dir'});
		$params{'dns'}                  = 1 if ($d->{'dns'});
		$params{'web'}                  = 1 if ($d->{'web'});
		$params{'virtualmin-nginx'}     = 1
			if ($d->{'virtualmin-nginx'});
		$params{'ssl'}                  = 1 if ($d->{'ssl'});
		$params{'virtualmin-nginx-ssl'} = 1
			if ($d->{'virtualmin-nginx-ssl'});
		$params{'mail'}                 = 1 if ($d->{'mail'});
		$params{'mysql'}                = 1 if ($d->{'mysql'});
		return \%params;
		};
	# Clone under the same docroot
	if ($target == 1) {
		$first_print->($text{"${tpref}clone$target"});
		my $clone_dir = $opts->{'clone_dir'};
		if ($clone_dir && $clone_dir !~ /^[\.\w-]+$/) {
			$second_print->($text{"${tpref}clone_edir"});
			}
		else {
			$clone->(&virtual_server::public_html_dir($d)."/$clone_dir", $d);
			}
		}
	# Existing but different domain
	elsif ($target == 2) {
		my $clone_dom = $opts->{'clone_dom'};
		my $clone_dom_dir = $opts->{'clone_dom_dir'};
		my $dom = &virtual_server::get_domain_by('id', $clone_dom);
		$dom = &virtual_server::get_domain_by('dom', $clone_dom) if (!$dom);
		if (!$dom) {
			$first_print->(&text("${tpref}clone$target",
					 $clone_dom));
			$second_print->($text{"${tpref}clone_eclonegone"});
			}
		else {
			$first_print->(&text("${tpref}clone$target",
					 $dom->{'dom'}));
			if (!@edoms) {
				$second_print->($text{"${tpref}clone_eclone"});
				}
			elsif ($clone_dom_dir &&
				  $clone_dom_dir !~ /^[\.\w-]+$/) {
				$second_print->($text{"${tpref}clone_edir"});
				}
			else {
				$clone->(&virtual_server::public_html_dir($dom).
						"/$clone_dom_dir", $dom);
				}
			}
		}
	# New sub-domain
	elsif ($target == 3) {
		my $clone_subdom_pref = $opts->{'clone_subdom_name'};
		$clone_subdom_pref = $clean_domain->($clone_subdom_pref);
		my $clone_subdom_dom = $opts->{'clone_subdom_dom'};
		my $clone_subdom_dir = $opts->{'clone_subdom_dir'};
		my $dom = &virtual_server::get_domain_by('id', $clone_subdom_dom);
		$dom = &virtual_server::get_domain_by('dom', $clone_subdom_dom) if (!$dom);
		my $clone_subdom_name;
		if ($dom) {
			$clone_subdom_name =
				"$clone_subdom_pref.$dom->{'dom'}";
			$first_print->(&text("${tpref}clone$target",
					 $clone_subdom_name));
			}
		if (!$dom) {
			$first_print->(&text("${tpref}clone$target",
				"$clone_subdom_pref.$clone_subdom_dom"));
			$second_print->($text{"${tpref}clone_eclonegone"});
			}
		elsif (!&virtual_server::can_create_sub_servers()) {
			$second_print->($text{"${tpref}clone_eclone"});
			}
		elsif ($clone_subdom_dir && $clone_subdom_dir !~ /^[\.\w-]+$/) {
			$second_print->($text{"${tpref}clone_edir"});
			}
		else {
			$indent_print->();
			$first_print->($text{"${tpref}clone_sub"});
			my $params = $cmd_params->();
			$params->{'parent'} = $dom->{'dom'};
			$params->{'plan'} =
				&virtual_server::get_plan($dom->{'plan'} // 0) ||
				&virtual_server::get_default_plan();
			$params->{'plan'} = $params->{'plan'}->{'name'};
			&reset_environment();
			$indent_print->();
			my $er = &virtual_server::create_domain_cli($clone_subdom_name, $params);
			$outdent_print->();
			&clean_environment();
			if (!ref($er)) {
				$er = lcfirst(&virtual_server::html_tags_to_text($er));
				$second_print->(&text("${tpref}clone_esub", $er));
				$outdent_print->();
				$second_print->($text{"${gpref}failed"});
				}
			else {
				$second_print->($text{'setup_done'});
				$outdent_print->();
				my $ndom = &virtual_server::get_domain_by('dom',
					$clone_subdom_name);
				$clone->(&virtual_server::public_html_dir($ndom).
					 "/$clone_subdom_dir", $ndom);
				}
			}
		}
	# New top-level domain
	elsif ($target == 4) {
		my $clone_newdom = $opts->{'clone_newdom'};
		$clone_newdom = $clean_domain->($clone_newdom);
		my $clone_newdom_dir = $opts->{'clone_newdom_dir'};
		$first_print->(&text("${tpref}clone$target", $clone_newdom));
		if (!&virtual_server::master_admin() &&
		    !&virtual_server::reseller_admin()) {
			$second_print->($text{"${tpref}clone_eclone"});
			}
		elsif ($clone_newdom_dir && $clone_newdom_dir !~ /^[\.\w-]+$/) {
			$second_print->($text{"${tpref}clone_edir"});
			}
		else {
			$indent_print->();
			$first_print->($text{"${tpref}clone_top"});
			my $params = $cmd_params->();
			$params->{'unix'} = 1;
			$params->{'pass'} = &substitute_pattern('[a-zA-Z0-9]{14}');
			&reset_environment();
			$indent_print->();
			my $er = &virtual_server::create_domain_cli($clone_newdom, $params);
			$outdent_print->();
			&clean_environment();
			if (!ref($er)) {
				$er = lcfirst(&virtual_server::html_tags_to_text($er));
				$second_print->(&text("${tpref}clone_etop", $er));
				$outdent_print->();
				$second_print->($text{"${gpref}failed"});
				}
			else {
				$second_print->($text{'setup_done'});
				$outdent_print->();
				my $ndom = &virtual_server::get_domain_by('dom',
					$clone_newdom);
				$clone->(&virtual_server::public_html_dir($ndom).
					 "/$clone_newdom_dir", $ndom);
				}
			}
		}
	}

# Ending block for general actions
if ($option_update) {
	if (!$bulk_type) {
		$outdent_print->();
		}
	$second_print->($updated_state ?
		$text{'setup_done'} : $text{'scripts_kit_wp_sapplying'});
	}

# Ending block for backup
if ($updated_state == 1 && $backup_tab) {
	$first_print->($text{"${tpref}backup_nothing"});
	$second_print->($text{'scripts_kit_wp_sapplying'});
	}

# Reset environment
&reset_environment();
}

sub script_wordpress_kit_cli
{
my ($argv) = @_;
# Parse params
my @argv = @{$argv};
my %opts;
while (@argv) {
	my $a = shift(@argv);
	if ($a eq '--script-path') {
		$opts{'spath'} = shift(@argv);
		}
	elsif ($a eq '--script-type') {
		$opts{'stype'} = shift(@argv);
		}
	elsif ($a eq '--domain') {
		$opts{'domain'} = shift(@argv);
		}
	elsif ($a eq '--wp-cli') {
		if (@argv && $argv[0] eq 'install') {
			$opts{'wp-cli'} = shift(@argv);
			}
		elsif (@argv && $argv[0] eq 'version') {
			$opts{'wp-cli'} = shift(@argv);
			}
		elsif (@argv && $argv[0] eq 'info') {
			$opts{'wp-cli'} = shift(@argv);
			}
		else {
			&usage("Invalid usage of --wp-cli parameter");
			}
		}
	# Parse the options flags
	elsif ($a eq '--opt') {
		my $opt_str = shift(@argv);
		# Parse the option string into key and value, handling
		# various quotes
		my $parse_opt_str = sub {
			my ($str) = @_;
			my @tokens;
			while ($str =~ /\G\s*(?:"((?:[^"\\]|\\.)*)"|
					'((?:[^'\\]|\\.)*)'|
					([^"\s]\S*))\s*/gx) {
				push(@tokens, (defined($1) ? $1 : 
						 (defined($2) ? $2 : $3)));
				}
			return @tokens if (@tokens);
			return;
			};
		# Parse the option string considering quotes
		my ($key, $value, $values) = $parse_opt_str->($opt_str);
		&usage("Invalid --opt format $opt_str")
			unless (defined($key) && defined($value));
		# Handle special options
		if ($key eq 'plugins') {
			my ($action, $plugins);
			if ($values) {
				($action, $plugins) = ($value, $values);
				}
			else {
				($action, $plugins) = $parse_opt_str->($value);
				}
			&usage("Invalid plugins option $value")
				unless (defined($action) && defined($plugins));
			if (exists($opts{'tab'}) && $opts{'tab'} ne 'plugins') {
				&usage("Cannot use other options with plugins");
				}
			$opts{'tab'} = 'plugins';
			$opts{'bulktype'} = 'plugin';
			$opts{'plugin'} = $action;
			my @plugins = split(/,/, $plugins);
			my @bulk;
			foreach my $plugin (@plugins) {
				push(@bulk, "$plugin\n$plugin");
				}
			$opts{'bulk'} = join("\0", @bulk);
			}
		elsif ($key eq 'themes') {
			my ($action, $themes);
			if ($values) {
				($action, $themes) = ($value, $values);
				}
			else {
				($action, $themes) = $parse_opt_str->($value);
				}
			&usage("Invalid themes option $value")
				unless (defined($action) && defined($themes));

			# Set the tab to 'themes' and ensure no mixing of tabs
			if (exists($opts{'tab'}) && $opts{'tab'} ne 'themes') {
				&usage("Cannot use other options with themes");
				}
			$opts{'tab'} = 'themes';
			$opts{'bulktype'} = 'theme';
			$opts{'theme'} = $action;
			my @themes = split(/,/, $themes);
			my @bulk;
			foreach my $theme (@themes) {
				push(@bulk, "$theme\n$theme");
				}
			$opts{'bulk'} = join("\0", @bulk);
			}
		elsif ($key eq 'backup') {
			my ($action, $backups);
			if ($values) {
				($action, $backups) = ($value, $values);
				}
			else {
				($action, $backups) = $parse_opt_str->($value);
				}
			unless (defined($action)) {
				&usage("Invalid backup option $value");
				}
			if (exists($opts{'tab'}) && $opts{'tab'} ne 'backup') {
				&usage("Cannot use other options with backup");
				}
			$opts{'tab'} = 'backup';
			$opts{'kit_constructor_backup'} = $action;
			if ($action eq 'restore' || $action eq 'delete') {
				my @files = $backups ? split(/,/, $backups) : ();
				$opts{'bulk'} = join("\0", @files) if @files;
				}
			}
		elsif ($key eq 'clone') {
			# Subroutine to parse cloning parameters
			my $parse_clone_params = sub {
				my ($str) = @_;
				my @params;
				while ($str =~ /--(\w+)=(".*?"|'.*?'|[^\s]+)/g) {
					my ($key, $value) = ($1, $2);
					$value =~ s/^["']|["']$//g;
					push(@params, [$key, $value]);
					}
				return @params;
				};

			# Handle cloning options
			&usage("Cannot use other options with clone")
				if (exists($opts{'tab'}) &&
				    $opts{'tab'} ne 'clone');
			$opts{'tab'} = 'clone';

			# Process cloning options based on action
			my %clone_map = (
				'same'     => 1,
				'existing' => 2,
				'sub'      => 3,
				'top'      => 4,
				);

			&usage("Missing cloning target for clone option")
			unless defined $value;

			if (exists $clone_map{$value}) {
				# Set cloning target
				$opts{'clone_target'} = $clone_map{$value};
				# Map cloning parameters to %opts keys
				my %clone_params_map = (
					# Same domain, new directory
					'dir'         => 'clone_dir',
					# Existing domain
					'dom'         => 'clone_dom',
					'dom_dir'     => 'clone_dom_dir',
					# New sub-domain
					'sub_name'    => 'clone_subdom_name',
					'sub'         => 'clone_subdom_dom',
					'sub_dir'     => 'clone_subdom_dir',
					# New top-level domain
					'top'         => 'clone_newdom',
					'top_dir'     => 'clone_newdom_dir',
					# Clone options
					'admin_email' => 'kit_clone_option_'.
							 'admin_email',
					'admin_pass'  => 'kit_clone_user_'.
							 'user_pass',
					'blogname'    => 'kit_clone_option_'.
							 'blogname',
					'siteurl'     => 'kit_clone_option_'.
							 'siteurl',
					);
				# Parse additional cloning parameters
				my @params = $parse_clone_params->($opt_str);
				foreach my $param (@params) {
					my ($p_key, $p_value) = @$param;
					if (exists($clone_params_map{$p_key})) {
						$opts{$clone_params_map{$p_key}} =
							$p_value;
						}
					else {
						&usage("Unknown cloning ".
							  "parameter: $p_key");
						}
					}
				}
				else {
					&usage("Unknown cloning target: $value");
					}
			}
		else {
			# Map option keys to prefixes
			my %option_map = (
				'wp_memory_limit'        => 'kit_const_',
				'wp_max_memory_limit'    => 'kit_const_',
				'fail2ban_foreign'       => 'kit_',
				'wp_auto_update_core'    => 'kit_const_',

				'siteurl'                => 'kit_option_',
				'home'                   => 'kit_option_',
				'blogname'               => 'kit_option_',
				'blogdescription'        => 'kit_option_',
				'permalink_structure'    => 'kit_option_',
				'uid'                    => '',
				'user_pass'              => 'kit_user_',
				'admin_email'            => 'kit_option_',
				'blog_public'            => 'kit_option_',
				'default_pingback_flag'  => 'kit_option_',
				'default_ping_status'    => 'kit_option_',
				'disallow_file_edit'     => 'kit_const_',

				'debug'                  => 'kit_constructor_',
				'maintenance'            => 'kit_constructor_',
				'disable_wp_cron'        => 'kit_const_',
				'concatenate_scripts'    => 'kit_const_',
				);
			if (exists($option_map{$key})) {
				$opts{"$option_map{$key}${key}"} = $value;
				}
			else {
				&usage("Unknown option key $key");
				}
			}
		}
	else {
		&usage("Unknown parameter $a");
		}
	}

# Test domain
$opts{'domain'} || &usage("Missing required --domain parameter");
my $d = &virtual_server::get_domain_by("dom", $opts{'domain'});
$d || &usage(&text('scripts_kit_edom_name', $opts{'domain'}));
&virtual_server::can_edit_domain($d) || &usage($text{'edit_ecannot'});
&virtual_server::can_edit_scripts() || &usage($text{'edit_ewebappcannot'});
&virtual_server::domain_has_website($d) && $d->{'dir'} || &usage($text{'scripts_eweb'});

# Get script info
my $sinfo;
my (@sinfos) = grep { $_->{'name'} eq $opts{'stype'} } &virtual_server::list_domain_scripts($d);
if (!@sinfos) {
	&usage(&text('scripts_kit_estype', $opts{'stype'}));
	}
elsif (scalar(@sinfos) != 1) {
	if ($opts{'spath'}) {
		($sinfo) =
		    grep { $_->{'opts'}->{'path'} eq $opts{'spath'} } @sinfos;
		$sinfo || &usage(&text('scripts_kit_estype_path',
				 $opts{'stype'}, $opts{'spath'}));
		}
	else {
		my @paths = map { $_->{'opts'}->{'path'} } @sinfos;
		@paths = sort { $a cmp $b } @paths;
		&usage(&text('scripts_kit_estype_multi', $opts{'stype'})."\n".
		join("\n", map { "    $_" } @paths)."\n".
		&text('scripts_kit_estype_multi2'));
		}
	}
else {
	$sinfo = $sinfos[0];
	}

# Get script
my $script = &virtual_server::get_script($sinfo->{'name'});

# Check, install, upgrade, or display instance info using WP-CLI
if ($opts{'wp-cli'}) {
	# Check existing command
	my $wpcli_cmd1 = "$d->{'home'}/bin/wp";
	my $wpcli_cmd2 = "$sinfo->{'opts'}->{'dir'}/wp-cli.phar";
	my $wpcli_cmd  = -f $wpcli_cmd1 ? $wpcli_cmd1 :
			 -f $wpcli_cmd2 ? $wpcli_cmd2 : undef;
	# Install WP-CLI
	if ($opts{'wp-cli'} eq 'install') {
		my $install_text = -r $wpcli_cmd ? "Reinstalling" : "Installing";
		print "$install_text WP-CLI ..\n";
		my $script_kit_wpcli_func = "script_$script->{'name'}_kit_wpcli";
		my $er = $script_kit_wpcli_func->($d, $sinfo);
		if ($er) {
			print ".. failed : @{[lcfirst($er)]}\n";
			}
		else {
			print ".. done : successfully install WP-CLI command\n";
			}
		}
	# Check WP-CLI version
	elsif ($opts{'wp-cli'} eq 'version') {
		print "Checking installed WP-CLI version ..\n";
		if ($wpcli_cmd) {
			my ($out, $err) = &virtual_server::run_as_domain_user(
				$d, "$wpcli_cmd --version 2>&1", undef, 1);
			if ($err) {
				print ".. failed : $out";
				exit($err >> 8);
				}
			else {
				$out =~ s/^/  /gm;
				$out =~ s/\n$//;
				print "$out\n.. done\n";
				}
			}
		else {
			print ".. error : WP-CLI is not installed\n";
			}
		}
	# Get instance info
	elsif ($opts{'wp-cli'} eq 'info') {
		print "Retrieving instance information ..\n";
		if (!-f $wpcli_cmd) {
			print ".. error : WP-CLI is not installed\n";
			return;
			}
		# Use WP-CLI command to get instance info
		my $site_dir = $sinfo->{'opts'}->{'dir'};
		my $wpcli_path_cmd = "$wpcli_cmd --path=$site_dir";
		my $run_wp_command = sub {
			my ($title, $cmd) = @_;
			print "  ==== $title ====\n";
			my ($out, $err) =
				&virtual_server::run_as_domain_user($d,
					"$cmd 2>&1", undef, 1);
			if ($err) {
				if ($out =~ s/^(Error|Warning|Notice):/      $1:/gm) {
					$out = "\n$out";
					}
				print "    Failed : $out";
				return 0;
				}
			else {
				$out =~ s/(?:PHP\s+)?(?:Warning|Deprecated|Notice):.*?\n//g;
				$out =~ s/^/    /gm;
				print "$out";
				return 1;
				}
			};
		my $display_formatted_list = sub {
			my ($title, $cmd) = @_;
			print "  ==== $title ====\n";
			my ($csv, $err) = &virtual_server::run_as_domain_user($d,
				"$cmd --format=csv 2>&1", undef, 1);
			if ($err) {
				$csv =~ s/\n/ /g;
				print "    Failed : $csv\n";
				return 0;
				}
			$csv =~ s/(?:PHP\s+)?(?:Warning|Deprecated|Notice):.*?\n//g;
			my @lines = split(/\n/, $csv);
			my @headers = split(/,/, shift(@lines));
			printf "    %-25s %-12s %-12s %-12s %-15s %-12s\n", @headers;
			foreach my $line (@lines) {
				my @fields = split(/,/, $line);
				$fields[0] = substr($fields[0], 0, 22) . '...'
					if (length($fields[0]) > 25);
				printf "    %-25s %-12s %-12s %-12s %-15s %-12s\n",
					@fields;
				}
			return 1;
			};
		# Using WP-CLI sub-commands
		$run_wp_command->("WORDPRESS CORE", "$wpcli_path_cmd core version --extra");
		if ($sinfo->{'name'} eq 'wordpress') {
			$run_wp_command->("CORE INTEGRITY", "$wpcli_path_cmd core verify-checksums");
			}
		$run_wp_command->("SITE CONFIGURATION", "echo -n 'Site URL: ' && \
			$wpcli_path_cmd option get siteurl && echo -n 'Home URL: ' && \
			$wpcli_path_cmd option get home");
		# Using WP-CLI eval command
		# Extended server environment info
		$run_wp_command->("SERVER ENVIRONMENT", "$wpcli_path_cmd eval '
			echo \"Server architecture: \" . php_uname(\"s\") . \" \" . php_uname(\"r\") . \" \" . php_uname(\"m\") . \"\\n\";
			echo \"Web server: \" . (isset(\$_SERVER[\"SERVER_SOFTWARE\"]) ? \$_SERVER[\"SERVER_SOFTWARE\"] : \"Unknown\") . \"\\n\";
			echo \"PHP version: \" . PHP_VERSION . \"\\n\";
			echo \"PHP SAPI: \" . php_sapi_name() . \"\\n\";
			echo \"PHP max input variables: \" . ini_get(\"max_input_vars\") . \"\\n\";
			echo \"PHP time limit: \" . ini_get(\"max_execution_time\") . \"\\n\";
			echo \"PHP memory limit: \" . ini_get(\"memory_limit\") . \"\\n\";
			echo \"Upload max filesize: \" . ini_get(\"upload_max_filesize\") . \"\\n\";
			echo \"PHP post max size: \" . ini_get(\"post_max_size\") . \"\\n\";
			echo \"cURL version: \" . (function_exists(\"curl_version\") ? curl_version()[\"version\"] . \" \" . curl_version()[\"ssl_version\"] : \"Not available\") . \"\\n\";
			echo \"Is SUHOSIN installed: \" . (extension_loaded(\"suhosin\") ? \"Yes\" : \"No\") . \"\\n\";
			echo \"Is the Imagick library available: \" . (extension_loaded(\"imagick\") ? \"Yes\" : \"No\") . \"\\n\";
			echo \"Current time: \" . date(\"c\") . \"\\n\";
			echo \"Current UTC time: \" . gmdate(\"l, d-M-y H:i:s T\") . \"\\n\";
		'");
		# Extended database information
		$run_wp_command->("DATABASE", "$wpcli_path_cmd eval '
			global \$wpdb;
			echo \"Database extension: mysqli\\n\";
			echo \"Server version: \" . \$wpdb->get_var(\"SELECT VERSION()\") . \"\\n\";
			echo \"Client version: mysqld \" . mysqli_get_client_info() . \"\\n\";
			echo \"Database username: \" . DB_USER . \"\\n\";
			echo \"Database password: \" . DB_PASSWORD . \"\\n\";
			echo \"Database host: \" . DB_HOST . \"\\n\";
			echo \"Database name: \" . DB_NAME . \"\\n\";
			echo \"Table prefix: \" . \$wpdb->prefix . \"\\n\";
			echo \"Database charset: \" . DB_CHARSET . \"\\n\";
		'");
		# Simplified database size query
		$run_wp_command->("DATABASE SIZE", "$wpcli_path_cmd db size --human-readable");
		# Directory sizes without database calculations
		$run_wp_command->("DIRECTORIES AND SIZES", "$wpcli_path_cmd eval '
			require_once(ABSPATH . \"wp-admin/includes/misc.php\");
			echo \"WordPress directory location: \" . ABSPATH . \"\\n\";
			echo \"WordPress directory size: \" . size_format(get_dirsize(ABSPATH)) . \"\\n\";
			echo \"Uploads directory location: \" . wp_upload_dir()[\"basedir\"] . \"\\n\";
			echo \"Uploads directory size: \" . size_format(get_dirsize(wp_upload_dir()[\"basedir\"])) . \"\\n\";
			echo \"Themes directory location: \" . WP_CONTENT_DIR . \"/themes\\n\";
			echo \"Themes directory size: \" . size_format(get_dirsize(WP_CONTENT_DIR . \"/themes\")) . \"\\n\";
			echo \"Plugins directory location: \" . WP_PLUGIN_DIR . \"\\n\";
			echo \"Plugins directory size: \" . size_format(get_dirsize(WP_PLUGIN_DIR)) . \"\\n\";
			echo \"Fonts directory location: \" . wp_upload_dir()[\"basedir\"] . \"/fonts\\n\";
			echo \"Fonts directory size: \" . (file_exists(wp_upload_dir()[\"basedir\"] . \"/fonts\") ? size_format(get_dirsize(wp_upload_dir()[\"basedir\"] . \"/fonts\")) : \"The directory does not exist.\") . \"\\n\";
		'");
		# WordPress constants
		$run_wp_command->("WORDPRESS CONSTANTS", "$wpcli_path_cmd eval '
			echo \"ABSPATH: \" . ABSPATH . \"\\n\";
			if (defined(\"WP_HOME\") && WP_HOME) echo \"WP_HOME: \" . WP_HOME . \"\\n\";
			if (defined(\"WP_SITEURL\") && WP_SITEURL) echo \"WP_SITEURL: \" . WP_SITEURL . \"\\n\";
			echo \"WP_CONTENT_DIR: \" . WP_CONTENT_DIR . \"\\n\";
			echo \"WP_PLUGIN_DIR: \" . WP_PLUGIN_DIR . \"\\n\";
			if (defined(\"WP_MEMORY_LIMIT\") && WP_MEMORY_LIMIT) echo \"WP_MEMORY_LIMIT: \" . WP_MEMORY_LIMIT . \"\\n\";
			if (defined(\"WP_MAX_MEMORY_LIMIT\") && WP_MAX_MEMORY_LIMIT) echo \"WP_MAX_MEMORY_LIMIT: \" . WP_MAX_MEMORY_LIMIT . \"\\n\";
			echo \"WP_DEBUG: \" . (defined(\"WP_DEBUG\") && WP_DEBUG ? \"Enabled\" : \"Disabled\") . \"\\n\";
			echo \"WP_DEBUG_DISPLAY: \" . (defined(\"WP_DEBUG_DISPLAY\") && WP_DEBUG_DISPLAY ? \"Enabled\" : \"Disabled\") . \"\\n\";
			echo \"WP_DEBUG_LOG: \" . (defined(\"WP_DEBUG_LOG\") && WP_DEBUG_LOG ? \"Enabled\" : \"Disabled\") . \"\\n\";
			echo \"SCRIPT_DEBUG: \" . (defined(\"SCRIPT_DEBUG\") && SCRIPT_DEBUG ? \"Enabled\" : \"Disabled\") . \"\\n\";
			echo \"WP_CACHE: \" . (defined(\"WP_CACHE\") && WP_CACHE ? \"Enabled\" : \"Disabled\") . \"\\n\";
			if (defined(\"CONCATENATE_SCRIPTS\") && CONCATENATE_SCRIPTS) echo \"CONCATENATE_SCRIPTS: True\\n\";
			if (defined(\"COMPRESS_SCRIPTS\") && COMPRESS_SCRIPTS) echo \"COMPRESS_SCRIPTS: True\\n\";
			if (defined(\"COMPRESS_CSS\") && COMPRESS_CSS) echo \"COMPRESS_CSS: True\\n\";
			if (defined(\"WP_ENVIRONMENT_TYPE\") && WP_ENVIRONMENT_TYPE) echo \"WP_ENVIRONMENT_TYPE: \" . WP_ENVIRONMENT_TYPE . \"\\n\";
			if (defined(\"WP_DEVELOPMENT_MODE\") && WP_DEVELOPMENT_MODE) echo \"WP_DEVELOPMENT_MODE: \" . WP_DEVELOPMENT_MODE . \"\\n\";
			echo \"DB_CHARSET: \" . DB_CHARSET . \"\\n\";
			if (DB_COLLATE) echo \"DB_COLLATE: \" . DB_COLLATE . \"\\n\";
		'");
		# File permissions
		$run_wp_command->("FILESYSTEM PERMISSIONS", "$wpcli_path_cmd eval '
			echo \"The main WordPress directory: \" . (wp_is_writable(ABSPATH) ? \"Writable\" : \"Not writable\") . \"\\n\";
			echo \"The wp-content directory: \" . (wp_is_writable(WP_CONTENT_DIR) ? \"Writable\" : \"Not writable\") . \"\\n\";
			echo \"The uploads directory: \" . (wp_is_writable(wp_upload_dir()[\"basedir\"]) ? \"Writable\" : \"Not writable\") . \"\\n\";
			echo \"The plugins directory: \" . (wp_is_writable(WP_PLUGIN_DIR) ? \"Writable\" : \"Not writable\") . \"\\n\";
			echo \"The themes directory: \" . (wp_is_writable(WP_CONTENT_DIR . \"/themes\") ? \"Writable\" : \"Not writable\") . \"\\n\";
			echo \"The fonts directory: \" . (file_exists(wp_upload_dir()[\"basedir\"] . \"/fonts\") ? wp_is_writable(wp_upload_dir()[\"basedir\"] . \"/fonts\") ? \"Writable\" : \"Not writable\" : \"Not writable\") . \"\\n\";
		'");
		# Display themes and plugins as formatted tables
		$display_formatted_list->("THEMES", "$wpcli_path_cmd theme list");
		$display_formatted_list->("PLUGINS", "$wpcli_path_cmd plugin list");
		# User counts and stats
		$run_wp_command->("STATISTICS", "$wpcli_path_cmd eval '
			echo \"Total users: \" . count_users()[\"total_users\"] . \"\\n\";
			echo \"Total posts: \" . wp_count_posts()->publish . \"\\n\";
			echo \"Total pages: \" . wp_count_posts(\"page\")->publish . \"\\n\";
			echo \"Total comments: \" . wp_count_comments()->total_comments . \"\\n\";
			echo \"Total approved comments: \" . wp_count_comments()->approved . \"\\n\";
			echo \"Total categories: \" . wp_count_terms(\"category\") . \"\\n\";
			echo \"Total tags: \" . wp_count_terms(\"post_tag\") . \"\\n\";
		'");
		# User roles
		$run_wp_command->("USER ROLES", "$wpcli_path_cmd eval '
			\$user_counts = count_users();
			foreach (\$user_counts[\"avail_roles\"] as \$role => \$count) {
				if (\$count > 0) {
					echo ucfirst(\$role) . \": \" . \$count . \" users\\n\";
				}
			}
			\$total_users = \$user_counts[\"total_users\"];
			\$users_with_roles = array_sum(\$user_counts[\"avail_roles\"]);
			\$users_without_roles = \$total_users - \$users_with_roles;
			if (\$users_without_roles > 0) {
				echo \"Users without a role: \" . \$users_without_roles . \"\\n\";
			}
		'");
		# Display WP-CLI info and version
		$run_wp_command->("WP-CLI INFO", "$wpcli_path_cmd cli info");
		print ".. done\n";
		}
	return;
	}

# Call apply
&virtual_server::set_all_text_print();
&script_wordpress_kit_apply($d, \%opts, $sinfo, $script);
}

sub script_wordpress_kit_wpcli
{
my ($d, $sinfo) = @_;
my $script_home = $sinfo->{'opts'}->{'dir'};
my $dom_home = $d->{'home'};
my $dom_bin = "$dom_home/bin";
my $old_wp_cli_path = "$script_home/wp-cli.phar";
my $wp_cli_path = "$dom_home/bin/wp";
&virtual_server::make_dir_as_domain_user($d, $dom_bin, 0755) if (!-d $dom_bin);
my $wp_cli_url =
    "https://raw.githubusercontent.com/wp-cli/builds/gh-pages/phar/wp-cli.phar";
# Delete possible old install
&virtual_server::unlink_file_as_domain_user($d, $old_wp_cli_path)
	if (-e $old_wp_cli_path && &is_under_directory($dom_home, $script_home));
# Download new WP-CLI
my $err;
my ($host, $port, $page, $ssl) = &parse_http_url($wp_cli_url);
&http_download($host, $port, $page, \$temp,
	       \$err, undef, 1, undef, undef, 5, undef, 1);
return &text('scripts_edownload', $err, "\"$wp_cli_url\"") if ($err);
eval { &virtual_server::write_as_domain_user($d,
	sub { &write_file_contents($wp_cli_path, $temp) }); };
return &text('scripts_kit_ewpewrite', $wp_cli_path, $@) if ($@);
&set_ownership_permissions($d->{'uid'}, $d->{'gid'}, 0750, $wp_cli_path);
return undef;
}

sub script_wordpress_kit_usage
{
my ($name) = @_;
my $s = ' ' x 27;
my $s2 = ' ' x 8;
print " | [--script-path path]\n";
print "$s --domain name\n";
print "$s --opt option | --wp-cli install|version|info\n";
print "\n";
print "  Available options for --opt:\n";
print "\n";

print "    Options:\n";
print "$s2 wp_memory_limit        256M|1G\n";
print "$s2 wp_max_memory_limit    512M|2G\n";
print "$s2 fail2ban_foreign       0|1\n";
print "$s2 wp_auto_update_core    false|minor|true\n\n";

print "$s2 siteurl                url\n";
print "$s2 home                   url\n";
print "$s2 blogname               blog_name\n";
print "$s2 blogdescription        blog_description\n";
print "$s2 permalink_structure    permalink_structure\n";
print "$s2 uid                    admin_user_id\n";
print "$s2 user_pass              admin_user_password\n";
print "$s2 admin_email            admin_email_address\n";
print "$s2 blog_public            0|1\n";
print "$s2 default_pingback_flag  0|1\n";
print "$s2 default_ping_status    open|closed\n";
print "$s2 disallow_file_edit     true|false\n\n";

print "$s2 debug                  off|on|log\n";
print "$s2 maintenance            activate|deactivate\n";
print "$s2 disable_wp_cron        true|false\n";
print "$s2 concatenate_scripts    true|false\n";
print "\n";
print "$s2 virtualmin configure-script --script-type $name \\\n";
print "$s2   --domain example.com \\\n";
print "$s2   --opt 'wp_memory_limit 512M' \\\n";
print "$s2   --opt 'blogname \"Blog Name\"'\n";
print "\n\n";

print "    Plugins options:\n";
print "$s2 plugins action plugin1,plugin2,...\n";
print "$s2   Actions: \n";
print "$s2     activate, deactivate, update, delete,\n";
print "$s2     enable-auto-updates, disable-auto-updates\n";
print "\n";
print "$s2 virtualmin configure-script --script-type $name \\\n";
print "$s2   --domain example.com \\\n";
print "$s2   --opt 'plugins activate plugin1,plugin2'\n";
print "\n\n";

print "    Themes options:\n";
print "$s2 themes action theme1,theme2,...\n";
print "$s2   Actions: \n";
print "$s2     activate, update, delete,\n";
print "$s2     enable-auto-updates, disable-auto-updates\n";
print "\n";
print "$s2 virtualmin configure-script --script-type $name \\\n";
print "$s2   --domain example.com \\\n";
print "$s2   --opt 'themes update theme1,theme2,theme3'\n";
print "\n\n";
print "    Backup options:\n";
print "$s2 backup action [filename1,filename2,...]\n";
print "$s2   Actions: full|files|db|restore|delete\n";
print "\n";
print "$s2 virtualmin configure-script --script-type $name \\\n";
print "$s2   --domain example.com --opt 'backup full'\n";
print "\n";
print "$s2 virtualmin configure-script --script-type $name \\\n";
print "$s2   --domain example.com \\\n";
print "$s2   --opt 'backup restore example.com_20241020-231846_full.tar.gz'\n";
print "\n\n";

print "    Clone options:\n";
print "$s2 clone target --param=value [--param=value ...]\n";
print "$s2   Targets:\n";
print "$s2     same        - clone to the same domain under a different directory\n";
print "$s2     existing    - clone to a different existing domain\n";
print "$s2     sub         - clone as a new subdomain\n";
print "$s2     top         - clone as a new top-level domain\n";
print "\n";
print "$s2   Parameters:\n";
print "$s2     dir         - directory for the clone (optional)\n\n";
print "$s2     dom         - existing domain name\n";
print "$s2     dom_dir     - directory under existing domain (optional)\n\n";
print "$s2     sub_dir     - directory under the new subdomain (optional)\n";
print "$s2     sub_name    - subdomain name\n";
print "$s2     sub         - parent domain name for the subdomain name\n\n";
print "$s2     top_dir     - directory under the new top-level domain (optional)\n";
print "$s2     top         - new top-level domain name\n\n";
print "$s2     admin_email - administrator email for the cloned site (optional)\n";
print "$s2     admin_pass  - administrator password for the cloned site (optional)\n";
print "$s2     blogname    - blog name for the cloned site (optional)\n";
print "$s2     siteurl     - site URL for the cloned site (optional)\n";
print "\n";
print "$s2 virtualmin configure-script --script-type $name --domain example.com \\\n";
print "$s2   --opt 'clone same --dir=$name-clone --blogname=\"Clone Name\"'\n";
print "\n";
print "$s2 virtualmin configure-script --script-type $name --domain example.com \\\n";
print "$s2   --opt 'clone existing --dom=otherdomain.com --dom_dir=$name'\n";
print "\n";
print "$s2 virtualmin configure-script --script-type $name --domain example.com \\\n";
print "$s2   --opt 'clone sub --sub_name=sub1 --sub=example.com \\\n";
print "$s2   --sub_dir=$name --admin_email=sub1\@example.com'\n";
print "\n";
print "$s2 virtualmin configure-script --script-type $name --domain example.com \\\n";
print "$s2   --opt 'clone top --top=example.net --top_dir=$name \\\n";
print "$s2   --admin_email=admin\@example.net --siteurl=https://www.example.net'\n";
print "\n";
print "\n";
print "  Available options for --wp-cli:\n\n";
print "    Options:\n";
print "$s2 install - install or reinstall WP-CLI command\n";
print "$s2 version - check installed WP-CLI version\n";
print "$s2 info    - get instance information\n";
print "\n";
print "$s2 virtualmin configure-script --script-type $name --domain example.com \\\n";
print "$s2   --wp-cli install\n";
print "$s2 virtualmin configure-script --script-type $name --domain example.com \\\n";
print "$s2   --wp-cli version\n";
print "$s2 virtualmin configure-script --script-type $name --domain example.com \\\n";
print "$s2   --wp-cli info\n";
}

# ClassicPress Kit (clone)
sub script_classicpress_kit_apply { return &script_wordpress_kit_apply(@_); }
sub script_classicpress_kit_usage { return &script_wordpress_kit_usage(@_); }
sub script_classicpress_kit_cli   { return &script_wordpress_kit_cli(@_); }
sub script_classicpress_kit_wpcli { return &script_wordpress_kit_wpcli(@_); }

1;
