
# script_wordpress_kit(&domain, &script, &opts)
# Called after a script is installed, to enable any extra actions needed
sub script_wordpress_kit
{
my ($d, $script, $sinfo) = @_;
my $mod = 'virtualmin-wp-workbench';
my $modpkg = $mod;
$modpkg =~ s/-/_/g;
my %mtext = &load_language($mod);
our %text = (%text, %mtext);
my $ohlink = \&hlink;
local *hlink = sub {
	my ($text, $page) = @_;
	$ohlink->($text, $page, $mod, undef, undef, { 'n' => $script->{'desc'} } );
	};
my $opts = $sinfo->{'opts'};

# Get actual PHP version for the script's directory
my (@phpdirs, $phpdir, $phpver);
@phpdirs = &virtual_server::list_domain_php_directories($d);
foreach (sort { length($a->{'dir'}) cmp length($b->{'dir'}) } @phpdirs) {
	if (&is_under_directory($_->{'dir'}, $opts->{'dir'}) ||
	    $_->{'dir'} eq $opts->{'dir'}) {
		$phpdir = $_; # Found the PHP directory
		}
	}
if ($phpdir) {
	$phpver = &virtual_server::get_php_version($phpdir->{'version'}, $d) ||
		  $phpdir->{'version'};
	$phpver =~ s/^(\d+\.\d+)\..*/$1/; # Get major version
	}
my $php = &virtual_server::get_php_cli_command(
	$phpver || $opts->{'phpver'}, $d) || &has_command("php");

# Create logs directory
my $logs_dir = "$d->{'home'}/logs";
if (!-d $logs_dir) {
	&make_dir_as_domain_user($d, "$logs_dir/logs", 0750);
	}
my $wp_cli_log = "$logs_dir/wp_cli_log";
my $backup_dir_name = $sinfo->{'backup_dir'} || "$script->{'name'}-backups";
my $wp_cli_path = -r "$d->{'home'}/bin/wp" ?
		     "$d->{'home'}/bin/wp" : 
		     "$opts->{'dir'}/wp-cli.phar";
my $wp_cli = "$php $wp_cli_path --path=$opts->{'dir'}";
my $_t = 'scripts_kit_wp_';
my $save_kit_form_pref = &get_webprefix()."/$mod";
my $kit_form_main = "data-form-nested='apply'";

# Can we run WP-CLI?
if (!-r $wp_cli_path) {
	my $html .= "$text{'scripts_kit_ewpfind'}\n<br>";
	$html .= &ui_link("../$mod/cli_installer.cgi?dom=$d->{'id'}&sid=$sinfo->{'id'}",
		&text('scripts_kit_ewpinstall'), 'btn btn-xxs btn-success');
	return $html;
	}

# Has to be called using eval for maximum speed (avg 
# expected load time is + 0.5s to default page load)
my $wp_cli_command = $wp_cli . ' eval \'
$backup_dir = "' . $d->{"home"} . '/' . $backup_dir_name . '";
if (!is_dir($backup_dir)) {
    mkdir($backup_dir, 0750, true);
}
$backups_data = array();
if (is_dir($backup_dir)) {
    $backup_files = scandir($backup_dir);
    foreach ($backup_files as $backup_file) {
        if ($backup_file !== "." && $backup_file !== ".." &&
	    preg_match("/\.tar\.gz$/", $backup_file)) {
            $backup_filepath = $backup_dir . "/" . $backup_file;
            if (!is_readable($backup_filepath)) {
                continue;
                }
            $size = filesize($backup_filepath);
            $creation_date = filectime($backup_filepath);
            $backups_data[] = array(
                "filename" => $backup_file,
                "filepath" => $backup_filepath,
                "size" => $size,
                "creation_date" => date("Y-m-d H:i:s", $creation_date),
            );
        }
    }
    usort($backups_data, function($a, $b) {
        return $b["creation_date"] <=> $a["creation_date"];
    });
}
$admin_user_email = get_option("admin_email");
$admin_id = get_user_by("email", $admin_user_email)->ID ?? 1;
$admin_user = get_userdata($admin_id)->user_login;
$admins = get_users([
    "role"    => "administrator",
    "orderby" => "ID",
    "order"   => "ASC"
]);
$admins_count = count($admins);
$admins_select_html = "<select id=\"admins_select\">\n";
foreach ($admins as $admin) {
    $admins_select_html .= "<option value=\"" . esc_attr($admin->ID) . "\">" . esc_html($admin->user_login) . "</option>\n";
}
$admins_select_html .= "</select>\n";
echo json_encode([
    "memory_limit" => ini_get("memory_limit"),
    "upload_max_filesize" => ini_get("upload_max_filesize"),
    "post_max_size" => ini_get("post_max_size"),
    "wp_debug" => defined("WP_DEBUG") ? WP_DEBUG : 0, 
    "wp_debug_display" => defined("WP_DEBUG_DISPLAY") ? WP_DEBUG_DISPLAY : 0, 
    "wp_debug_log" => defined("WP_DEBUG_LOG") ? str_replace("' . $opts->{'dir'} . '", "", WP_DEBUG_LOG) : 0, 
    "wp_memory_limit" => defined("WP_MEMORY_LIMIT") ? WP_MEMORY_LIMIT : 0, 
    "wp_max_memory_limit" => defined("WP_MAX_MEMORY_LIMIT") ? WP_MAX_MEMORY_LIMIT : 0, 
    "disallow_file_edit" => defined("DISALLOW_FILE_EDIT") ? (DISALLOW_FILE_EDIT === false ? "false" : "true") : "false", 
    "concatenate_scripts" => defined("CONCATENATE_SCRIPTS") ? (CONCATENATE_SCRIPTS === false ? "false" : "true") : "true", 
    "disable_wp_cron" => defined("DISABLE_WP_CRON") ? (DISABLE_WP_CRON === true ? "true" : "false") : "false", 
    "wp_auto_update_core" => defined("WP_AUTO_UPDATE_CORE") ? (WP_AUTO_UPDATE_CORE === false ? "false" : (WP_AUTO_UPDATE_CORE === "minor" ? "minor" : "true")) : "minor",
    "automatic_updater_disabled" => defined("AUTOMATIC_UPDATER_DISABLED") ? (AUTOMATIC_UPDATER_DISABLED == true ? 1 : 0) : 0,
    "blogname" => get_option("blogname"), 
    "blog_public" => get_option("blog_public"), 
    "default_pingback_flag" => get_option("default_pingback_flag"), 
    "default_ping_status" => get_option("default_ping_status"), 
    "maintenance" => file_exists(ABSPATH . ".maintenance") == true ? "activate" : "deactivate",
    "admins_count" => $admins_count,
    "admin_id" => $admin_id,
    "admin_user" => $admin_user,
    "admin_email" => $admin_user_email,
    "admins_select" => $admins_select_html,
    "version" => get_bloginfo("version"),
    "blogdescription" => get_option("blogdescription"),
    "url" => get_bloginfo("url"),
    "wpurl" => get_bloginfo("wpurl"),
    "language" => get_bloginfo("language"),
    "wordpress_version" => get_bloginfo("version"),
    "php_version" => phpversion(),
    "permalink_structure" => get_option("permalink_structure"),
    "permalinks" => [
        "plain" => [
                "label" => "Plain",
                "structure" => "",
                "example" => "/?p=123"
        ],
        "day_and_name" => [
                "label" => "Day and name",
                "structure" => "/%year%/%monthnum%/%day%/%postname%/",
                "example" => "/2024/05/12/sample-post/"
        ],
        "month_and_name" => [
                "label" => "Month and name",
                "structure" => "/%year%/%monthnum%/%postname%/",
                "example" => "/2024/05/sample-post/"
        ],
        "numeric" => [
                "label" => "Numeric",
                "structure" => "/archives/%post_id%",
                "example" => "/archives/123"
        ],
        "post_name" => [
                "label" => "Post name",
                "structure" => "/%postname%/",
                "example" => "/sample-post/"
        ]
    ],
    "login_url" => [admin_url(), get_bloginfo("url")],
    "plugins" => array_map(function($plugin) {
        require_once(ABSPATH . "wp-admin/includes/plugin.php");
        require_once(ABSPATH . "wp-admin/includes/update.php");
        $data = get_plugin_data(WP_PLUGIN_DIR . "/" . $plugin);
        $update = get_site_transient("update_plugins");
        $auto_updates = (array) get_site_option("auto_update_plugins", array());
            $isAutoUpdateEnabled = in_array($plugin, $auto_updates);
        $new_version = 0;
        if (isset($update->response[$plugin])) {
                if (is_object($update->response[$plugin])) {
                        $new_version = $update->response[$plugin]->new_version;
                } elseif (is_array($update->response[$plugin])) {
                        $new_version = $update->response[$plugin]["new_version"];
                }
        }
        return [
            "name" => $data["Name"],
            "dir" =>  dirname(plugin_basename($plugin)),
            "description" => $data["Description"],
            "version" => $data["Version"],
            "new_version" => $new_version,
            "active" => is_plugin_active($plugin) ? 1 : 0,
            "auto_update" => $isAutoUpdateEnabled ? 1 : 0,
            "reqphp" => $data["RequiresPHP"],
            "reqwp" => $data["RequiresWP"],
        ];
    }, array_keys(get_plugins())),
    "themes" => array_map(function($theme) {
        require_once(ABSPATH . "wp-admin/includes/theme.php");
        require_once(ABSPATH . "wp-admin/includes/update.php");
        $theme_data = wp_get_theme($theme);
        $current_theme = wp_get_theme()->get_stylesheet();
        $isActive = ($theme === $current_theme);
        $update = get_site_transient("update_themes");
        $auto_updates = (array) get_site_option("auto_update_themes", array());
            $isAutoUpdateEnabled = in_array($theme, $auto_updates);
        $new_version = 0;
        if (isset($update->response[$theme])) {
                if (is_object($update->response[$theme])) {
                $new_version = $update->response[$theme]->new_version;
                } elseif (is_array($update->response[$theme])) {
                $new_version = $update->response[$theme]["new_version"];
                }
        }
        return [
            "name" => $theme_data->get("Name"),
	    "dir" =>  $theme_data->get_stylesheet(),
            "description" => $theme_data->get("Description"),
            "version" => $theme_data->get("Version"),
            "new_version" => $new_version,
            "active" => $isActive ? 1 : 0,
            "auto_update" => $isAutoUpdateEnabled ? 1 : 0,
        ];
    }, array_keys(wp_get_themes())),
    "backups_data" => $backups_data,
]);\'';
&clean_environment();
my ($wp, $wperr) = &run_as_domain_user($d, "$wp_cli_command 2>> $wp_cli_log");
&reset_environment();
if ($wperr) {
	$wperr = &html_escape(&text('scripts_kit_ewprun', $wp_cli_log));
	return "<pre>$wperr</pre>";
	}
eval { $wp = &convert_from_json($wp); };
$wp = &{"${modpkg}::encode_json_strings"}($wp);
if ($@) {
	&error_stderr("Failed to parse JSON output from WP-CLI command : $@");
	my $err = "$text{'scripts_kit_ewp'} : $@";
	$wp = &text('scripts_kit_elog', "`$wp_cli`", $wp_cli_log);
	$err .= ($err =~ /\n$/ ? $wp : " : $wp");
	return "<pre>@{[&html_escape($err)]}</pre>";
	}

# Store original 
my $wpj = { %$wp,
	    plugins => [ map { { %$_ } } @{$wp->{'plugins'}} ],
	    themes  => [ map { { %$_ } } @{$wp->{'themes'}} ] };
map { delete $_->{'description'} } @{$wpj->{'plugins'}};
map { delete $_->{'description'} } @{$wpj->{'themes'}};
my $debug_mode = $wp->{'wp_debug'} ? ($wp->{'wp_debug_log'} ? 2 : 1) : 0;
$wpj->{'opt_wp_debug'} = $debug_mode;
$wpj = &convert_to_json($wpj);

# Form abstraction
my $form = sub {
	my ($tab) = @_;
	my $form = &ui_form_start("$save_kit_form_pref/workbench.cgi",
			   "post", undef, "data-form-nested='apply' id='kit_${tab}_form'");
	return $form;
	};
# Form definitions abstraction
my $form_defs = sub {
	my ($tab) = @_;
	my $form_defs = &ui_hidden("dom", $d->{'id'});
	$form_defs .= &ui_hidden("tab", $tab) if ($tab);
	$form_defs .= &ui_hidden("sid", $sinfo->{'id'});
	$form_defs .= &ui_hidden("uid", $wp->{'admin_id'});
	$form_defs .= &ui_hidden("sstate", $wpj);
	return $form_defs;
	};

# Tabs list
my @tabs = (
	[ "system", 'System' ],
	[ "settings", 'Settings' ],
	[ "plugins", 'Plugins' ],
	[ "themes", 'Themes' ],
	[ "backup", 'Backup and Restore' ],
	&can_edit_databases() ? [ "clone", 'Clone' ] : ( ),
	[ "development", 'Development' ] );

# Do we have tab in URL
my $tab = $in{'tab'};

# Validate if passed tab in URL is in
# tabs list or fall back to default
$tab = $tabs[1]->[0] if (!$tab || !grep { $_->[0] eq $tab } @tabs);

# System tab prepare
my $system_tab_content;
# Memory limit
push(@$system_tab_content, {
	desc  => &hlink($text{"${_t}memory_limit"},
			      "kit_wp_memory_limit"),
	value => &ui_opt_textbox(
	    "kit_const_wp_memory_limit", undef, 6,
	    	"$text{\"${_t}memory_limit_upto\"} $wp->{'wp_memory_limit'}",
		$text{'edit_set'})});
# Admin memory limit
push(@$system_tab_content, {
	desc  => &hlink($text{"${_t}max_memory_limit"},
			      "kit_wp_max_memory_limit"),
	value => &ui_opt_textbox(
	    "kit_const_wp_max_memory_limit", undef, 6,
		"$text{\"${_t}memory_limit_upto\"} ".
			($wp->{'wp_max_memory_limit'} == -1 ? '256M' : 
			 $wp->{'wp_max_memory_limit'}),
		$text{'edit_set'})});
# Fail2Ban protection
if (&virtual_server::master_admin() && &foreign_available("fail2ban") &&
    &foreign_check("fail2ban") && &foreign_installed("fail2ban")) {
	&foreign_require("fail2ban");
	my $conf_dir = $fail2ban::config{'config_dir'};
	my $name = "$script->{'name'}-$d->{'dom'}";
	$name =~ s/\./-/g;
	my $ffile = "$conf_dir/filter.d/$name.conf";
	my $jfile = "$conf_dir/jail.d/99-$name.conf";
	my $conf_exists = -r $ffile || -r $jfile? 1 : 0;
	my $wp = &convert_from_json($wpj);
	$wp = &{"${modpkg}::encode_json_strings"}($wp);
	$wp->{'fail2ban'} = $conf_exists;
	$wpj = &convert_to_json($wp);
	push(@$system_tab_content, {
		desc  => &hlink($text{"${_t}fail2ban"}, "kit_wp_fail2ban"),
		value => &ui_yesno_radio(
		    "kit_fail2ban_foreign", $conf_exists) });
	}
# Site automatic updates
push(@$system_tab_content, {
	desc  => &hlink($text{"${_t}auto_update_core"},
	                "kit_wp_auto_update_core"),
	value => &ui_radio(
	    "kit_const_wp_auto_update_core", $wp->{'wp_auto_update_core'} ,
		[ [ 'false', $text{"${_t}auto_updates_disabled"} . "<br>" ],
		  [ 'minor', $text{"${_t}auto_updates_minor"} . "<br>" ],
		  [ 'true', $text{"${_t}auto_updates_major_minor"} ] ] )});

# Settings tab prepare
my $settings_tab_content;
# Site URL
push(@$settings_tab_content, {
	desc  => &hlink($text{"${_t}url"}, "kit_wp_url"),
	value => &ui_opt_textbox(
	    "kit_option_siteurl", undef, 35, $wp->{'wpurl'} . "<br>",
	    $text{'edit_set'})});
# Home
push(@$settings_tab_content, {
	desc  => &hlink($text{"${_t}wpurl"}, "kit_wp_wpurl"),
	value => &ui_opt_textbox(
	    "kit_option_home", undef, 35, $wp->{'url'} . "<br>",
	    $text{'edit_set'})});
# Site name
push(@$settings_tab_content, {
	desc  => &hlink($text{"${_t}blogname"}, "kit_wp_blogname"),
	value => &ui_opt_textbox(
	    "kit_option_blogname", undef, 25, $wp->{'blogname'} . "<br>",
	    $text{'edit_set'})});
# Site blogdescription
push(@$settings_tab_content, {
	desc  => &hlink($text{"${_t}blogdescription"},
			"kit_wp_blogdescription"),
	value => &ui_opt_textbox(
	    "kit_option_blogdescription", undef, 35,
	    $wp->{'blogdescription'} || $text{"scripts_kit_not_set"} . "<br>",
	    $text{'edit_set'}) });
# Permalink structure
my $permlink_structure_opts = [
	[ $wp->{'permalinks'}->{'plain'}->{'structure'},
	  $wp->{'permalinks'}->{'plain'}->{'label'} ],
	[ $wp->{'permalinks'}->{'day_and_name'}->{'structure'},
	  $wp->{'permalinks'}->{'day_and_name'}->{'label'} ],
	[ $wp->{'permalinks'}->{'month_and_name'}->{'structure'},
	  $wp->{'permalinks'}->{'month_and_name'}->{'label'} ],
	[ $wp->{'permalinks'}->{'post_name'}->{'structure'},
	  $wp->{'permalinks'}->{'post_name'}->{'label'} ] ];
# If custom permalink structure is not in the list, add it
if (!grep { $_->[0] eq $wp->{'permalink_structure'} } @$permlink_structure_opts) {
	push(@$permlink_structure_opts,
		[ $wp->{'permalink_structure'},
		  $text{"${_t}permalink_structure_custom"} . 
		  	" [$wp->{'permalink_structure'}]" ]);
	}
push(@$settings_tab_content, {
	desc  => &hlink($text{"${_t}permalink_structure"},
			"kit_wp_permalink_structure"),
	value => &ui_select("kit_option_permalink_structure",
			$wp->{'permalink_structure'},
			$permlink_structure_opts) });
# Admin username
if (!$wp->{'admins_count'}) {
	push(@$settings_tab_content, {
		desc  => &hlink($text{"${_t}admin_username_missing"},
				      "kit_wp_admin_username_missing"),
		value => &ui_submit($text{"${_t}admin_username_setup_admin"},
					  'kit_constructor_add_admin')});
	}
elsif ($wp->{'admins_count'} == 1) {
	push(@$settings_tab_content, {
		desc  => &hlink($text{"${_t}admin_username"},
				      "kit_wp_admin_username"),
		value => $wp->{'admin_user'}});
	}
elsif ($wp->{'admins_count'} > 1) {
	push(@$settings_tab_content, {
		desc  => &hlink($text{"${_t}admin_usernames"},
				      "kit_wp_admin_usernames"),
		value => $wp->{'admins_select'}});
	}

# Set password
push(@$settings_tab_content, {
	desc  => &hlink($text{"${_t}user_pass"}, "kit_wp_user_pass"),
	value => &ui_opt_textbox(
	    "kit_user_user_pass", undef, 20, $text{'user_passdef'},
	    $text{'edit_set'})});
# Admin email
push(@$settings_tab_content, {
	desc  => &hlink($text{"${_t}admin_email"}, "kit_wp_admin_email"),
	value => &ui_opt_textbox(
	    "kit_option_admin_email", undef, 30, $wp->{'admin_email'} . "<br>",
	    $text{'edit_set'})});
# Site visibility
push(@$settings_tab_content, {
	desc  => &hlink($text{"${_t}blog_public"}, "kit_wp_blog_public"),
	value => &ui_yesno_radio(
	    "kit_option_blog_public", $wp->{'blog_public'} ? 1 : 0)});
# Pingbacks
push(@$settings_tab_content, {
	desc  => &hlink($text{"${_t}default_pingback_flag"},
			      "kit_wp_default_pingback_flag"),
	value => &ui_yesno_radio(
	    "kit_option_default_pingback_flag",
	    	$wp->{'default_pingback_flag'} ? 1 : 0)});
# Trackbacks
push(@$settings_tab_content, {
	desc  => &hlink($text{"${_t}default_ping_status"},
			      "kit_wp_default_ping_status"),
	value => &ui_yesno_radio(
	    "kit_option_default_ping_status",
	    	$wp->{'default_ping_status'} eq 'open' ? 'open' : 'closed',
			'open', 'closed')});
# File editing
push(@$settings_tab_content, {
	desc  => &hlink($text{"${_t}disallow_file_edit"},
			      "kit_wp_disallow_file_edit"),
	value => &ui_yesno_radio(
	    "kit_const_disallow_file_edit", $wp->{'disallow_file_edit'},
	    	'true', 'false')});

# Plugins tab prepare
my $plugins_tab_content;
my $plugins_actions_opts =
	[ [ "", $text{"${_t}selopt_bulk"} ],
	  [ "activate", $text{"${_t}selopt_activate"} ],
	  [ "deactivate", $text{"${_t}selopt_deactivate"} ],
	  [ "update", $text{"${_t}selopt_update"} ],
	  [ "delete", $text{"${_t}selopt_delete"} ],
	  [ "enable-auto-updates", $text{"${_t}selopt_enable_auto"} ],
	  [ "disable-auto-updates", $text{"${_t}selopt_disable_auto"} ] ];
$plugins_tab_content = $form->('plugins');
$plugins_tab_content .= $form_defs->('plugins');
$plugins_tab_content .= &ui_hidden("bulktype", "plugin");
$plugins_tab_content .= &ui_select("plugin", "", $plugins_actions_opts);
$plugins_tab_content .= &ui_submit($text{'scripts_kit_apply'}, "apply");
$plugins_tab_content .= &ui_submit($text{'scripts_kit_updcache'},
				   "kit_constructor_flushcache");
$plugins_tab_content .= &ui_submit(&text('scripts_kit_openin_wp', $script->{'desc'}),
				   "kit_form_login_plugins", undef,
				   "form='kit_login_form'");
$plugins_tab_content .= &ui_columns_start(
	[ "", $text{"${_t}tb_plugin"},
	      $text{"${_t}tb_installed_version"},
	      $text{"${_t}tb_update_available"},
	      $text{"${_t}tb_enabled"},
	      $text{"${_t}tb_auto_update"},
	], 100, 0, [ ( "width=5" ) ]);
foreach my $plugin (@{$wp->{'plugins'}}) {
	my $plugin_desc = $plugin->{'description'};
	if ($plugin_desc) {
		$plugin_desc = " " .
			&ui_help(&html_escape(
				&html_strip($plugin_desc)));
		}
	$plugins_tab_content .= &ui_checked_columns_row([
		&html_escape($plugin->{'name'}, 1).$plugin_desc,
		&html_escape($plugin->{'version'}),
		$plugin->{'new_version'} ?
			"<b>".&ui_text_color(&html_escape(
				$plugin->{'new_version'}), 'success')."</b>" :
			$text{'no'},
		$plugin->{'active'} ? $text{'yes'} : $text{'no'},
		$plugin->{'auto_update'} ? $text{'yes'} : $text{'no'},
	], [ ( "width=5" ) ], 'bulk',
		&quote_escape("$plugin->{'dir'}\n$plugin->{'name'}", '"'));
}
$plugins_tab_content .= &ui_columns_end();
$plugins_tab_content .= &ui_form_end();

# Themes tab prepare
my $themes_tab_content;
splice(@$plugins_actions_opts, 2, 1);
$themes_tab_content = $form->('themes');
$themes_tab_content .= $form_defs->('themes');
$themes_tab_content .= &ui_hidden("bulktype", "theme");
$themes_tab_content .= &ui_select("theme", "", $plugins_actions_opts);
$themes_tab_content .= &ui_submit($text{'scripts_kit_apply'}, "apply");
$themes_tab_content .= &ui_submit($text{'scripts_kit_updcache'},
				  "kit_constructor_flushcache");
$themes_tab_content .= &ui_submit(&text('scripts_kit_openin_wp', $script->{'desc'}),
				  "kit_form_login_themes", undef,
				  "form='kit_login_form'");
$themes_tab_content .= &ui_columns_start(
	[ "", $text{"${_t}tb_theme"},
	      $text{"${_t}tb_installed_version"},
	      $text{"${_t}tb_update_available"},
	      $text{"${_t}tb_active"},
	      $text{"${_t}tb_auto_update"},
	], 100, 0, [ ( "width=5" ) ]);
foreach my $theme (@{$wp->{'themes'}}) {
	my $theme_desc = $theme->{'description'};
	if ($theme_desc) {
		$theme_desc = " " .
			&ui_help(&html_escape(
				&html_strip($theme_desc)));
		}
	$themes_tab_content .= &ui_checked_columns_row([
		&html_escape($theme->{'name'}, 1).$theme_desc,
		&html_escape($theme->{'version'}),
		$theme->{'new_version'} ?
			"<b>".&ui_text_color(&html_escape(
				$theme->{'new_version'}), 'success')."</b>" :
			$text{'no'},
		$theme->{'active'} ? $text{'yes'} : $text{'no'},
		$theme->{'auto_update'} ? $text{'yes'} : $text{'no'},
	], [ ( "width=5" ) ], 'bulk',
		&quote_escape("$theme->{'dir'}\n$theme->{'name'}", '"'));
}
$themes_tab_content .= &ui_columns_end();
$themes_tab_content .= &ui_form_end();

# Backup and restore tab prepare
my $backup_tab_content;
my $backup_actions_opts =
	[ [ "", $text{"${_t}selopt_bulk"} ],
	  [ "full", $text{"${_t}selopt_backup_all"} ],
	  [ "files", $text{"${_t}selopt_backup_files"} ],
	  [ "db", $text{"${_t}selopt_backup_db"} ],
	  [ "restore", $text{"${_t}selopt_backup_restore"} ],
	  [ "delete", $text{"${_t}selopt_backup_del"} ] ];
$backup_tab_content .= $form->('backup');
$backup_tab_content .= $form_defs->('backup');
$backup_tab_content .= &ui_select("kit_constructor_backup", "",
	$backup_actions_opts);
$backup_tab_content .= &ui_submit($text{'scripts_kit_apply'}, "apply");
my $backup_content_files;
my $backup_content_files_size_all = 0;
foreach my $backup_data (@{$wp->{'backups_data'}}) {
        if ($backup_data->{'size'}) {
                $backup_content_files_size_all += $backup_data->{'size'};
		$backup_type = $backup_data->{'filename'};
		$backup_type =~ s/.*?_\S+-\S+_(full|db|files)\.tar.gz$/$1/;
		my $f =
		     "$d->{'home'}/$backup_dir_name/$backup_data->{'filename'}";
		$backup_data->{'creation_date'} = strftime("%Y-%m-%d %H:%M:%S",
			localtime((stat($f))[10]));
                $backup_content_files .= &ui_checked_columns_row([
                        &html_escape($backup_data->{'creation_date'}),
                        &nice_size(&html_escape($backup_data->{'size'})),
			&html_escape($text{"scripts_kit_backup_tb_type_".
				$backup_type}),
                        &html_escape($backup_data->{'filename'})
                ], [ ( "width=5" ) ], 'bulk',
			&quote_escape($backup_data->{'filename'}, '"'));
                }
        }
$backup_tab_content .= &ui_table_start(undef, "width=100%", 2);
$backup_tab_content .= &ui_table_row(
		$text{'scripts_kit_backups_location'}, "~/$backup_dir_name", 2);
$backup_tab_content .= &ui_table_row(
		$text{'scripts_kit_backups_size'},
			&nice_size($backup_content_files_size_all), 2);
$backup_tab_content .= &ui_table_end();
if ($backup_content_files) {
        $backup_tab_content .= &ui_columns_start(
                [ "",
		$text{"scripts_kit_backup_tb_date"},
                $text{"scripts_kit_backup_tb_size"},
		$text{"scripts_kit_backup_tb_type"},
                $text{"scripts_kit_backup_tb_filename"},
                ], 100, 0, [ ( "width=5" ) ]);
        $backup_tab_content .= $backup_content_files;
        $backup_tab_content .= &ui_columns_end();
        }
$backup_tab_content .= &ui_form_end();

# Clone tab prepare
my $clone_tab_content = $form->('clone');
$clone_tab_content .= $form_defs->('clone');
$clone_tab_content .= &ui_table_start(undef, "width=100%", 2);
my $slink = $sinfo->{'url'};
$slink =~ s/^https?:\/\///;
$slink =~ s/\/$//;
$clone_tab_content .=
	&ui_table_row($text{'scripts_kit_clone_source1'}, $slink, 2);
$clone_tab_content .=
	&ui_table_row($text{'scripts_kit_clone_source2'}, $opts->{'dir'}, 2);
my $clone_target;
my @vdoms = sort { lc($a->{'dom'}) cmp lc($b->{'dom'}) }
	    grep { !$_->{'alias'} } &list_visible_domains();
my @edoms = grep {  $_->{'dom'} ne $d->{'dom'} } @vdoms;
my $doms_select = sub {
	my ($name, $doms) = @_;
	&ui_select($name, undef,
		[ map { [ $_->{'id'}, &show_domain_name($_) ] } @{$doms} ]);
	};
my $opts_path_placeholder = "$opts->{'path'}";
$opts_path_placeholder =~ s/\///;
$opts_path_placeholder ||= $sinfo->{'name'};
$opts_path_placeholder .= '-clone';
$clone_target = &ui_radio_table("clone_target", 1,
	[ [ 1, $text{'scripts_kit_clone_target1'},
		&ui_textbox("clone_dir", undef, 15, undef, undef,
			"placeholder='$opts_path_placeholder'") ],
	  @edoms ? 
	  	[ 2, $text{'scripts_kit_clone_target2'},
		     $doms_select->('clone_dom', \@edoms)."&nbsp;/&nbsp;&nbsp;".
		     &ui_textbox("clone_dom_dir", undef, 15, undef, undef,
		     	"placeholder='$opts_path_placeholder'")] : (),
	  @vdoms && &can_create_sub_servers() ?
	  	[ 3, $text{'scripts_kit_clone_target3'},
			&ui_textbox("clone_subdom_name", undef, 5, undef, undef,
				"placeholder='sub1'")."&nbsp;.&nbsp;&nbsp;".
				$doms_select->('clone_subdom_dom', \@vdoms).
				"&nbsp;/&nbsp;&nbsp;".
			&ui_textbox("clone_subdom_dir", undef, 15, undef, undef,
				"placeholder='$opts_path_placeholder'") ] : (),
	  (&virtual_server::master_admin() || &virtual_server::reseller_admin()) ? 
		[ 4, $text{'scripts_kit_clone_target4'},
		     &ui_textbox("clone_newdom", "", 20). "&nbsp;/&nbsp;&nbsp;".
		     &ui_textbox("clone_newdom_dir", undef, 15, undef, undef,
		     	"placeholder='$opts_path_placeholder'") ] : (),
	]);
$clone_tab_content .= &ui_table_row(
	&hlink($text{'scripts_kit_clone_target'},
		"kit_target_cloned"), $clone_target, 2);
$clone_tab_content .= &ui_table_row(
	&hlink($text{"${_t}url_cloned"}, "kit_wp_url_cloned"),
		&ui_opt_textbox(
			"kit_clone_option_siteurl", undef, 35,
			$text{'scripts_kit_auto'} . "<br>",
		$text{'edit_set'}), 2);
$clone_tab_content .= &ui_table_row(
	&hlink($text{"${_t}blogname_cloned"}, "kit_wp_blogname_cloned"),
		&ui_opt_textbox(
			"kit_clone_option_blogname", undef, 25,
			$text{'scripts_kit_nochange'} . "<br>",
		$text{'edit_set'}), 2);
$clone_tab_content .= &ui_table_row(
	&hlink($text{"${_t}admin_email_cloned"}, "kit_wp_admin_email_cloned"),
		&ui_opt_textbox(
			"kit_clone_option_admin_email", undef, 30,
			$text{'scripts_kit_nochange'} . "<br>",
		$text{'edit_set'}), 2);
$clone_tab_content .= &ui_table_row(
	&hlink($text{"${_t}user_pass_cloned"}, "kit_wp_user_pass_cloned"),
		&ui_opt_textbox(
			"kit_clone_user_user_pass", undef, 20,
			$text{'scripts_kit_nochange'} . "<br>",
		$text{'edit_set'}), 2);
$clone_tab_content .= &ui_table_end();
$clone_tab_content .= &ui_form_end();

# Development tab prepare
my $development_tab_content;
# Debug mode
push(@$development_tab_content, {
	desc  => &hlink($text{"${_t}debug"}, "kit_wp_wp_debug"),
	value => &ui_radio(
	    "kit_constructor_debug", $debug_mode,
		[ [ 0, $text{"${_t}debug0"} . "<br>" ],
		  [ 1, $text{"${_t}debug1"} . "<br>" ],
		  [ 2, $text{"${_t}debug2"} ] ] )});
# Maintenance mode
push(@$development_tab_content, {
	desc  => &hlink($text{"${_t}maintenance_mode"},
			      "kit_wp_maintenance_mode"),
	value => &ui_yesno_radio(
	    "kit_constructor_maintenance", $wp->{'maintenance'},
	    	'activate', 'deactivate')});
# Disable cron jobs
push(@$development_tab_content, {
	desc  => &hlink($text{"${_t}disable_cron"}, "kit_wp_disable_cron"),
	value => &ui_yesno_radio(
	    "kit_const_disable_wp_cron", $wp->{'disable_wp_cron'},
	    	'true', 'false')});
# Script concatenation
push(@$development_tab_content, {
	desc  => &hlink($text{"${_t}concatenate_scripts"},
			      "kit_wp_concatenate_scripts"),
	value => &ui_yesno_radio(
	    "kit_const_concatenate_scripts", $wp->{'concatenate_scripts'},
	    	'true', 'false')});

# All tabs start
my $data = &ui_tabs_start(\@tabs, "tab", $tab, 0);

# System tab content
$data .= &ui_tabs_start_tab("tab", "system");
$data .= &vui_ui_block(&text('scripts_kit_wp_system_desc', $script->{'desc'}));
$data .= $form->('system');
$data .= $form_defs->('system');
$data .= &ui_table_start(undef, "width=100%", 2);
foreach my $option (@$system_tab_content) {
	$data .= &ui_table_row($option->{'desc'}, $option->{'value'});
	}
$data .= &ui_table_end();
$data .= &ui_form_end();
$data .= &ui_tabs_end_tab();

# Settings tab content
my @data_submits;
$data .= &ui_tabs_start_tab("tab", "settings");
$data .= &vui_ui_block(&text('scripts_kit_wp_settings_desc', $script->{'desc'}));
$data .= $form->('settings');
$data .= $form_defs->('settings');
$data .= &ui_table_start(undef, "width=100%", 2);
foreach my $option (@$settings_tab_content) {
	$data .= &ui_table_row($option->{'desc'}, $option->{'value'});
	}
$data .= &ui_table_end();
push(@data_submits, &ui_submit($text{'scripts_kit_apply'},
	"kit_form_apply", undef,
	"data-submit-nested='apply' form='kit_${tab}_form'"));
push(@data_submits, &ui_submit(&text('scripts_kit_wp_login', $script->{'desc'}),
	"kit_form_login", undef, "form='kit_login_form'"));
$data .= &ui_form_end();
$data .= &ui_tabs_end_tab();

# Plugins tab content
$data .= &ui_tabs_start_tab("tab", "plugins");
$data .= &vui_ui_block(&text('scripts_kit_wp_plugins_desc', $script->{'desc'}));
$data .= $plugins_tab_content;
$data .= &ui_tabs_end_tab();

# Themes tab content
$data .= &ui_tabs_start_tab("tab", "themes");
$data .= &vui_ui_block(&text('scripts_kit_wp_themes_desc', $script->{'desc'}));
$data .= $themes_tab_content;
$data .= &ui_tabs_end_tab();

# Backup and restore tab content
$data .= &ui_tabs_start_tab("tab", "backup");
$data .= &vui_ui_block(&text('scripts_kit_wp_backup_desc', $script->{'desc'}));
$data .= $backup_tab_content;
$data .= &ui_tabs_end_tab();

# Clone tab content
if (&can_edit_databases()) {
	$data .= &ui_tabs_start_tab("tab", "clone");
	$data .= &vui_ui_block(&text('scripts_kit_wp_clone_desc', $script->{'desc'}));
	$data .= $clone_tab_content;
	$data .= &ui_tabs_end_tab();
	}

# Development tab content
$data .= &ui_tabs_start_tab("tab", "development");
$data .= &vui_ui_block(&text('scripts_kit_wp_development_desc', $script->{'desc'}));
$data .= $form->('development');
$data .= $form_defs->('development');
$data .= &ui_table_start(undef, "width=100%", 2);
foreach my $option (@$development_tab_content) {
	$data .= &ui_table_row($option->{'desc'}, $option->{'value'});
	}
$data .= &ui_table_end();
$data .= &ui_form_end();
$data .= &ui_form_start("$save_kit_form_pref/script_login.cgi",
			"post", undef, "id='kit_login_form' target='_blank'");
$data .= $form_defs->();
$data .= &ui_form_end();
$data .= &ui_tabs_end_tab();

# All tabs end
$data .= &ui_tabs_end();

# Print JS script to handle multiple admins select
$data .= <<EOF;
<script>
(function() {
	const select = document.getElementById("admins_select");
	if (select) {
		select.addEventListener("change", function() {
			const forms = document.querySelectorAll('form[action*="script_login.cgi"], form[action*="workbench.cgi"]');
			forms.forEach((form) => {
				form.uid.value = this.value;
			});
			const up = 'wp-' + btoa(location.origin).slice(0, 32) +
				'-admin-user';
			localStorage.setItem(up, this.value);
		});
	}
})();
</script>
EOF

# If the admin was previously selected, select it in the form on page reload
$data .= <<EOF;
<script>
(function() {
	const up = localStorage.getItem('wp-' +
		btoa(location.origin).slice(0, 32) + '-admin-user') ||
		'$in{'auid'}';
	const select = document.getElementById("admins_select");
	if (select && up) {
		select.value = up;
		select.dispatchEvent(new Event('change'));
	}
})();
</script>
EOF

return { extra_submits => \@data_submits, data => $data };
}

# script_wordpress_kit_login(&domain, &script, &script-info, &script-current-data, &post-data)
# Called to login to the WordPress admin panel
sub script_wordpress_kit_login
{
my ($d, $script, $sinfo, $sstate, $post) = @_;
my $esdesc = "$script->{'desc'} $text{'scripts_kit_loginkit'}";
my $login_data = $sstate->{'login_url'};
my $login_uid = $post->{'uid'};
$login_uid =~ /^\d+$/ ||
	&error("$esdesc : $text{'scripts_kit_einvaliduid'} : $login_uid");
my $admin_url = $login_data->[0];
$admin_url =~ /:\/\// ||
	&error("$esdesc : $text{'scripts_kit_einvalidadminurl'} : $admin_url");
my $site_url = $login_data->[1];
$site_url =~ /:\/\// ||
	&error("$esdesc : $text{'scripts_kit_einvalidsiteurl'} : $site_url");
$admin_url .= "plugins.php" if ($post->{'kit_form_login_plugins'});
$admin_url .= "themes.php" if ($post->{'kit_form_login_themes'});
my $dir = $sinfo->{'opts'}->{'dir'};
my $filename = "/wp-login-".&substitute_pattern('[a-f0-9]{40}').".php";
my $dir_filename = "$dir/$filename";
$dir_filename =~ s/([^:])\/\//$1\//g;
my $redir_url = "$site_url$filename";
$redir_url =~ s/([^:])\/\//$1\//g;
my $fcontents = <<EOF;
<?php
require __DIR__ . '/wp-load.php';
register_shutdown_function(function() {
    unlink('$dir_filename');
});
\$current_user_id = get_current_user_id();
if (\$current_user_id != $login_uid) {
	wp_clear_auth_cookie();
	wp_set_current_user($login_uid);
	wp_set_auth_cookie($login_uid);
}
wp_redirect('$admin_url');
exit;
EOF
&write_as_domain_user($d, sub { 
	&write_file_contents($dir_filename, $fcontents) });
&redirect($redir_url);
}

1;